/**********************************************************************
Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
***********************************************************************/

#include "mol.h"
#include "oeutil.h"
#include "parsmart.h"
#include "typer.h"
#include "rotor.h"
#include "binary.h"
#include "commandline.h"
#include "version.h"
#include "data.h"

#include <stdio.h>

#ifdef __sgi
#include <iostream.h>
#include <fstream.h>
#else
#include <iostream>
#include <fstream>
#endif

using namespace std;
using namespace OpenEye;

void usage();

// There isn't a great way to do this -- we need to save argv[0] for usage()
static char *program_name;

int main(int argc,char *argv[])
{
  io_type inFileType, outFileType;
  OEFileFormat fileFormat;

  program_name = argv[0];
  if (argc != 3 || !argv[1] || !argv[2])
    {
      usage();
      exit (-1);
    }

  inFileType = extab.FilenameToType(argv[1]);
  outFileType = extab.FilenameToType(argv[2]);

  OEMol mol(inFileType, outFileType);
  ifstream inFileStream(argv[1]);
  ofstream outFileStream(argv[2]);

  if (!inFileStream)
    {
      cerr << program_name << ": Cannot read input file!" << endl;
      exit (-1);
    }

  if (!outFileStream)
    {
      cerr << program_name << ": Cannot write to output file!" << endl;
      exit (-1);
    }

  fileFormat.ReadMolecule(inFileStream,mol);
  fileFormat.WriteMolecule(outFileStream,mol);

  return(0);
}

void usage()
{
unsigned int i;

  cout << "Babel " << BABEL_VERSION << " -- " << __DATE__ << " -- "
       << __TIME__ << endl;
  cout << "Usage is : " << endl << program_name 
       << " <input name> <output name>" << endl;
  cout << endl << "Currently supported input types" << endl;
  for (i = 0; i < extab.Count(); i++)
    if (extab.IsReadable(i))
      cout << "\t" << extab.GetExtension(i) << " -- " 
		<< extab.GetDescription(i) << " file" << endl;
  cout << endl << "Currently supported output types" << endl;
  for (i = 0; i < extab.Count(); i++)
    if (extab.IsWritable(i))
      cout << "\t" << extab.GetExtension(i) << " -- " 
		<< extab.GetDescription(i) << " file" << endl;

  exit(0);
}
