/**********************************************************************
Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
***********************************************************************/

#include <vector>
#include <stdio.h>
#include <math.h>

using namespace std;

void print_matrix(vector<vector<float> > &m)
{
  unsigned int i,j;
  
  for (i = 0; i < m.size(); i++)
  {
    for (j = 0; j < m[i].size(); j++)
      printf("%5.2f",m[i][j]);
    printf("\n");
  }
}

bool mult_matrix(vector<vector<float> > &c, vector<vector<float> > &a, vector<vector<float> > &b)
{
  unsigned int i,j,k;
  
  if (a.size() != b.size())
    return(false);

  c.resize(a.size());
  
  for (i = 0; i < a.size(); i++)
    {
      c[i].resize(b[i].size());
      for (j = 0; j < b[i].size(); j++)
	{
	  c[i][j] = 0.0;
	  for (k = 0; k < a[i].size(); k++)
	    {
	      c[i][j] = c[i][j] + a[i][k] * b[k][j];
	    }
	}
    }
  return(true);
}


bool invert_matrix(vector<vector<float> > &mat, float &det)
{
  int  i, j, k, m, n, row = 0, col = 0;
  double tempo, big, pvt;
  vector<int> pvt_ind;
  vector<vector<int> > index;
  int cols = mat[0].size();
  int rows = mat.size();
  
  pvt_ind.resize(mat[0].size());

  index.resize(mat.size());
  for (i = 0; (unsigned)i < mat.size(); i++)
    index[i].resize(2);

  // make sure we have a square matrix
  // #rows == #cols;
  if (cols != rows)
  {
    det = 0.0;
    return(false);
  }
  
  det = 1.0;
  for (i = 0; i < cols; i++)
    pvt_ind[i] = rows+1;
  for (i = 0; i < cols; i++)
  {
    big = 0.0;
    for (j = 0; j < cols; j++)
    {
      if (pvt_ind[j] != 0)
	for (k = 0; k < cols; k++)
	{
	  if (fabs(big) < fabs(mat[j][k]))
	  {
	    row = j;
	    col = k;
	    big = mat[j][k];
	  }
	}
    }
    pvt_ind[col]++;
    if (row != col)
    {
      det = -det;
      for (m = 0; m < cols; m++)
      {
	tempo = mat[row][m];
	mat[row][m] = mat[col][m];
	mat[col][m] = tempo;
      }
    }
    index[i][0] = row;
    index[i][1] = col;
    pvt = mat[col][col];
    det *= pvt;
    
    mat[col][col] = 1.0;
    for (m = 0; m < cols; m++)
      mat[col][m] /= pvt;
    for (n = 0; n < cols; n++)
      if (n != col)
      {
	tempo = mat[n][col];
	mat[n][col] = 0.0;
	for (m = 0; m < cols; m++)
	  mat[n][m] -= mat[col][m] * tempo;
      }
  }
  
  for (i = 0; i < cols; i++)
  {
    m = cols - 1;
    if (index[m][0] != index[m][1])
    {
      row = index[m][0];
      col = index[m][1];
      for (k = 0; k < cols; k++)
      {
	tempo = mat[k][row];
	mat[k][row] = mat[k][col];
	mat[k][col] = tempo;
      }
    }
  }
  return(true);
}


    
      
    
	    
  

  

