/**********************************************************************
Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
***********************************************************************/

#ifndef _PARSMART_H
#define _PARSMART_H

/*==========================*/
/*  SMARTS Data Structures  */
/*==========================*/

#define AE_LEAF      0x01
#define AE_RECUR     0x02
#define AE_NOT       0x03
#define AE_ANDHI     0x04
#define AE_OR        0x05
#define AE_ANDLO     0x06

#define AL_CONST     0x01
#define AL_MASS      0x02
#define AL_AROM      0x03
#define AL_ELEM      0x04
#define AL_HCOUNT    0x05
#define AL_NEGATIVE  0x06
#define AL_POSITIVE  0x07
#define AL_CONNECT   0x08
#define AL_DEGREE    0x09
#define AL_IMPLICIT  0x0a
#define AL_RINGS     0x0b
#define AL_SIZE      0x0c
#define AL_VALENCE   0x0d
#define AL_CHIRAL    0x0e
#define AL_HYB       0x0f
#define AL_CLOCKWISE     1
#define AL_ANTICLOCKWISE 2

namespace OpenEye {

typedef union _AtomExpr {
        int type;
        struct {
            int type;
            int prop;
            int value;
        } leaf;
        struct {
            int type;
            void *recur;
        } recur;
        struct {
            int type;
            union _AtomExpr *arg;
        } mon;
        struct {
            int type;
            union _AtomExpr *lft;
            union _AtomExpr *rgt;
        } bin;
    } AtomExpr;

#define BE_LEAF      0x01
#define BE_ANDHI     0x02
#define BE_ANDLO     0x03
#define BE_NOT       0x04
#define BE_OR        0x05

#define BL_CONST     0x01
#define BL_TYPE      0x02

#define BT_SINGLE     0x01
#define BT_DOUBLE     0x02
#define BT_TRIPLE     0x03
#define BT_AROM       0x04
#define BT_UP         0x05
#define BT_DOWN       0x06
#define BT_UPUNSPEC   0x07
#define BT_DOWNUNSPEC 0x08
#define BT_RING       0x09

typedef union _BondExpr {
        int type;
        struct {
            int type;
            int prop;
            int value;
        } leaf;
        struct {
            int type;
            union _BondExpr *arg;
        } mon;
        struct {
            int type;
            union _BondExpr *lft;
            union _BondExpr *rgt;
        } bin;
    } BondExpr;

typedef struct {
        BondExpr *expr;
        int src,dst;
        int visit;
        bool grow;
    } BondSpec;

typedef struct {
  AtomExpr *expr;
  int visit;
  int part;
  int chiral_flag;
  int vb;
    } AtomSpec;

typedef struct {
  int aalloc,acount;
  int balloc,bcount;
  bool ischiral;
  AtomSpec *atom;
  BondSpec *bond;
  int parts;
} Pattern;

class OESmartsPattern
{
 protected:
  vector<vector<int> > _mlist;
  Pattern *_pat;
  vector<bool> _growbond;
  string _str;
 public:
  OESmartsPattern()  {_pat=NULL;}
  ~OESmartsPattern();
  int   NumMatches()               {return(_mlist.size());}
  int   NumAtoms()                 {return((_pat) ? _pat->acount:0);}
  int   NumBonds()                 {return((_pat) ? _pat->bcount:0);}
  int   GetVectorBinding(int idx)  {return(_pat->atom[idx].vb);}
  int   GetCharge(int);
  int   GetAtomicNum(int);
  void  WriteMapList(ostream&);
  void  GetBond(int&,int&,int&,int);
  bool IsValid() {return(_pat != (Pattern*)NULL);}
  bool Init(char*);
  bool Init(string&);
  bool Empty()                     {return(_pat == NULL);}
  bool Match(OEMol&,bool single=false);
  bool RestrictedMatch(OEMol&,vector<pair<int,int> >&,bool single=false);
  bool RestrictedMatch(OEMol&,OEBitVec&,bool single=false);
  vector<vector<int> > &GetMapList() {return(_mlist);}
  vector<vector<int> > &GetUMapList();
  vector<vector<int> >::iterator BeginMList() {return(_mlist.begin());}
  vector<vector<int> >::iterator EndMList() {return(_mlist.end());}
  string &GetSMARTS() {return(_str);}
};

class OESSMatch //used for fast exhaustive matching
{
protected:
  bool                *_uatoms;
  OEMol               *_mol;
  Pattern             *_pat;
  vector<int>          _map;
public:
  OESSMatch(OEMol&,Pattern*);
  ~OESSMatch();
  void Match(vector<vector<int> >&,int bidx=-1);
};

void SmartsLexReplace(string&,vector<pair<string,string> >&);



}

#endif //_PARSMART_H

