// ERRUTIL.H : Handle simple error messages and Yes/No problems
//             will use GNOME/GTK dialogue boxes if available.

// Copyright (C) 2000 Geoffrey Hutchison

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#ifndef ERRUTIL_H
#define ERRUTIL_H

/*################################################################################################*/

#include "config.h"

class err_util;
class console_err_util;

class err_util_factory;
class console_err_util_factory;

/*################################################################################################*/

/**	Various user-visible error routines depending on severity. These may be 
	overloaded by subclasses (e.g. GUI dialogues).
*/

class err_util
{
	protected:
	
	public:
	
	err_util(void) { }
	virtual ~err_util(void) { }
	
	virtual void Message(const char *) = 0;
	virtual void WarningMessage(const char *) = 0;
	virtual void ErrorMessage(const char *) = 0;
	
	virtual bool Question(const char *) = 0;
};

class console_err_util : public err_util
{
	protected:
	
	public:
	
	console_err_util(void);
	~console_err_util(void);
	
	void Message(const char *);		// virtual
	void WarningMessage(const char *);	// virtual
	void ErrorMessage(const char *);	// virtual
	
	bool Question(const char *);		// virtual
};

/*################################################################################################*/

/**	The purpose of this is to "##produce" err_util objects. This is a generic way to 
	create the objects, whether we need base class or derived class objects.
*/

class err_util_factory
{
	protected:
	
	public:
	
	err_util_factory(void) { }
	virtual ~err_util_factory(void) { }
	
	virtual err_util * Produce(void) = 0;
};

class console_err_util_factory : public err_util_factory
{
	private:
	
	static console_err_util_factory * instance;
	console_err_util_factory(void);
	
	public:
	
	~console_err_util_factory(void);
	static console_err_util_factory * GetInstance(void);
	
	err_util * Produce(void);	// virtual
};

/*################################################################################################*/

#endif	// ERRUTIL_H

// eof
