// MM1MDL.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "mm1mdl.h"
#include "v3d.h"

#include "mm1eng1.h"
#include "mm1eng9.h"

#include "filetrans.h"

#include <fstream>
#include <strstream>

#include <algorithm>

/*################################################################################################*/

mm1_sequencebuilder mm1_mdl::amino_builder(MM1_AMINO_BUILDER_FILE);
mm1_sequencebuilder mm1_mdl::nucleic_builder(MM1_NUCLEIC_BUILDER_FILE);

mm1_mdl::mm1_mdl(ostream * p1, err_util_factory & p2) : model_extended(p1, p2), model_simple(p1, p2)
{
	nmol = NOT_DEFINED;
	ref_civ = NULL;
	
	trajfile = NULL;
	
	default_eng = 0; periodic = false;
	box_hdim[0] = 0.5; box_fdim[0] = 2.0 * box_hdim[0];
	box_hdim[1] = 1.0; box_fdim[1] = 2.0 * box_hdim[1];
	box_hdim[2] = 1.5; box_fdim[2] = 2.0 * box_hdim[2];
}

mm1_mdl::~mm1_mdl(void)
{
}

const char * mm1_mdl::GetProjectFileNameExtension(void)
{
	static const char ext[] = "mm1gp";
	return ext;
}

/*##############################################*/
/*##############################################*/

const char * mm1_mdl::engtab1[] =
{
	"mm1_eng_exp9 : the normal engine",
	"mm1_eng_exp1_mim : the periodic engine (minimum image model)",
	NULL
};

const i32s mm1_mdl::engtab2[] =
{
	ENG_MM1_NORMAL,
	ENG_MM1_PERIODIC,
	NOT_DEFINED
};

mm1_eng * mm1_mdl::CreateDefaultEngine(void)
{
	switch (engtab2[default_eng])
	{
		case ENG_MM1_PERIODIC:
		periodic = true; GatherGroups();
		return new mm1_eng_exp1_mim((* this), 1, 0);
		
		default:	// ENG_MM1_NORMAL
		periodic = false;
		return new mm1_eng_exp9((* this), 1, 0);
	}
}

/*##############################################*/
/*##############################################*/

void mm1_mdl::PushCRDSets(i32u p1)
{
//	cs_vector.resize(cs_vector.size() + p1);			// more convenient way?!?!?!
//	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back();		// more crash-proof way?!?!?!?
	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());	// or even this way?!?!?!?
	
	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32u n1 = 0;n1 < p1;n1++)
		{
			(* it1).crd_vector.push_back(newcrd);
		}
	}
}

void mm1_mdl::PopCRDSets(i32u)
{
}

void mm1_mdl::CopyCRDSet(i32u, i32u)
{
}

void mm1_mdl::SwapCRDSets(i32u, i32u)
{
}

void mm1_mdl::CenterCRDSet(i32u p1)
{
	fGL sum[3] = { 0.0, 0.0, 0.0 };
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++) sum[n1] += (* it1).crd_vector[p1][n1];
	}
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			fGL tmp1 = sum[n1] / (f64) atom_list.size();
			(* it1).crd_vector[p1][n1] -= tmp1;
		}
	}
}

void mm1_mdl::ReserveCRDSets(i32u)
{
}

void mm1_mdl::AddAtom(mm1_atom & p1)
{
	InvalidateGroups();
	atom_list.push_back(p1);
}

void mm1_mdl::RemoveAtom(iter_mm1al it1)
{
	InvalidateGroups();
	
	// this strange while-loop is needed because removing bonds
	// will invalidate all those atom::cr_list-iterators...
	
	while ((* it1).cr_list.begin() != (* it1).cr_list.end())
	{
		mm1_cr * ref = & (* it1).cr_list.back();
		iter_mm1bl it2 = find(bond_list.begin(), bond_list.end(), (* ref->bndr));
		
		if (it2 != bond_list.end()) RemoveBond(it2);
		else if (ostr != NULL) (* ostr) << "bad news: find failed in RA" << endl;
	}
	
	atom_list.erase(it1);
}

void mm1_mdl::AddBond(mm1_bond & p1)
{
	InvalidateGroups();
	bond_list.push_back(p1);
	
	mm1_cr info1 = mm1_cr(p1.atmr[1], & bond_list.back());
	p1.atmr[0]->cr_list.push_back(info1);
	
	mm1_cr info2 = mm1_cr(p1.atmr[0], & bond_list.back());
	p1.atmr[1]->cr_list.push_back(info2);
}

void mm1_mdl::RemoveBond(iter_mm1bl it1)
{
	InvalidateGroups();
	mm1_cr tmpinfo = mm1_cr(NULL, & (* it1));
	iter_mm1cl it2;
	
	it2 = find((* it1).atmr[0]->cr_list.begin(), (* it1).atmr[0]->cr_list.end(), tmpinfo);
	if (it2 != (* it1).atmr[0]->cr_list.end()) (* it1).atmr[0]->cr_list.erase(it2);
	else if (ostr != NULL) (* ostr) << "bad news: find #1 failed in RB" << endl;
	
	it2 = find((* it1).atmr[1]->cr_list.begin(), (* it1).atmr[1]->cr_list.end(), tmpinfo);
	if (it2 != (* it1).atmr[1]->cr_list.end()) (* it1).atmr[1]->cr_list.erase(it2);
	else if (ostr != NULL) (* ostr) << "bad news: find #2 failed in RB" << endl;
	
	bond_list.erase(it1);
}

bool mm1_mdl::ReadStream_OLD(istream & p1)
{
	static const char message[] = "Hmmm. It seems that we are trying to read a VERY OLD version of this file.\nIt's not possible to verify the type and version of the file, so this\noperation may fail. Are you sure you wish to try (Y/N) ??? ";
	
	bool result = err->Question(message);
	if (!result) return false;
	
	vector<mm1_atom *> tmp1; i32s tmp2[2]; char tmp3; fGL tmp4[3];
	
#ifdef VERBOSE
cout << "mm1_mdl::ReadStream_OLD() : starting file input." << endl;
#endif	// VERBOSE

	while (p1.peek() != 'E')
	{
		p1 >> tmp2[0] >> tmp2[1];
		p1 >> tmp4[0] >> tmp4[1] >> tmp4[2] >> tmp2[0]; p1.get();
		
		element el = element(tmp2[1]);
		mm1_atom newatom(el, tmp4, cs_vector.size());
		AddAtom(newatom); tmp1.push_back(& atom_list.back());
		
		for (i32s n1 = 0;n1 < tmp2[0];n1++)
		{
			p1 >> tmp2[1] >> tmp3;
			if (tmp2[1] < (i32s) tmp1.size())
			{
				mm1_bond newbond(tmp1[tmp2[1]], tmp1[tmp1.size() - 1], bondtype(tmp3));
				AddBond(newbond);
			}
		}
		
		p1.getline(buffer, sizeof(buffer));
	}
	
	return true;
}

bool mm1_mdl::ReadStream(istream & p1)
{
	int version;
	
	// first we have to identify the file version, and then call the matching function,
	// if necessary. the very first file format did not use the "!Section" idea, so
	// we must skip the test if no such traces are seen...
	
	if (p1.peek() != '!') version = -1;
	else
	{
		// now we expect to see "!Header" section.
		// next we will check the file type and version information...
		
		p1 >> buffer;		// read "!Header".
		p1 >> buffer;		// read the file type.
		p1 >> version;		// read the version number;
		
		// just return false if new have incorrect file type...
		
		//		if (strcmp(buffer, GetProjectFileNameExtension()))
		//		{
		//#ifdef VERBOSE
		//cout << "mm1_mdl::ReadStream() : incorrect file type : " << buffer << " != " << GetProjectFileNameExtension() << endl;
		//#endif	// VERBOSE
		//
		//			return false;
		//		}
		
		p1.getline(buffer, sizeof(buffer));
	}
	
	// now look at the version number, and pick a correct function if needed.
	
	if (version < 100) return ReadStream_OLD(p1);
	else if (version > 100)
	{
		static const char message[] = "Ooops. It seems that we are trying to read a FUTURE version of this file.\nIt's not sure if we can handle that. Are you sure you wish to try (Y/N) ??? ";
		
		bool result = err->Question(message);
		if (!result) return false;
	}
	
	// the "!Header" section is now processed, and the actual file input begins!!!
	// the "!Header" section is now processed, and the actual file input begins!!!
	// the "!Header" section is now processed, and the actual file input begins!!!
	
#ifdef VERBOSE
cout << "mm1_mdl::ReadStream() : starting file input." << endl;
#endif	// VERBOSE

	i32u crd_set_counter = 0;
	vector<mm1_atom *> ar_vector;
	
	// now we just read the sections, until we hit into "!End"-section and leave the loop...
	
	while (true)
	{
		p1 >> buffer;		// read the section string.
		
		if (!strcmp(buffer, "!End")) break;		// handle "!End" section.
		else if (!strcmp(buffer, "!Info"))		// handle "!Info" section.
		{
			int n1; p1 >> n1;
			
			int n2 = n1 - cs_vector.size();
			if (n2 > 0)
			{
				i32u n3 = cs_vector.size(); PushCRDSets((i32u) n2);
				for (i32u n4 = n3;n4 < cs_vector.size();n4++) cs_vector[n4].visible = true;
			}
			
#ifdef VERBOSE
cout << "handled \"!Info\" section, added " << n2 << " coordinate sets." << endl;
#endif	// VERBOSE

			p1.getline(buffer, sizeof(buffer));
			
			// TODO : read coordinate set descriptions (if there are some)...
			// TODO : read coordinate set descriptions (if there are some)...
			// TODO : read coordinate set descriptions (if there are some)...
		}
		else if (!strcmp(buffer, "!Atoms"))		// handle "!Atoms" section.
		{
			p1.getline(buffer, sizeof(buffer));
			
#ifdef VERBOSE
cout << "found \"!Atoms\" section..." << endl;
#endif	// VERBOSE

			while (p1.peek() != '!')
			{
				i32u n1; p1 >> n1;	// id number
				i32s n2; p1 >> n2;	// atomic number
				
#ifdef VERBOSE
if (n1 != ar_vector.size()) cout << "WARNING : bad atom id numbers!!!" << endl;
#endif	// VERBOSE

				element tmp_element(n2);
				mm1_atom newatom(tmp_element, NULL, cs_vector.size());
				
				AddAtom(newatom);
				ar_vector.push_back(& atom_list.back());
				
				p1.getline(buffer, sizeof(buffer));
			}
		}
		else if (!strcmp(buffer, "!Bonds"))		// handle "!Bonds" section.
		{
			p1.getline(buffer, sizeof(buffer));
			
#ifdef VERBOSE
cout << "found \"!Bonds\" section..." << endl;
#endif	// VERBOSE

			while (p1.peek() != '!')
			{
				i32u id1; p1 >> id1;	// 1st id number
				i32u id2; p1 >> id2;	// 2nd id number
				
				char bt; p1 >> bt;	// bond type
				
#ifdef VERBOSE
if (id1 >= ar_vector.size()) cout << "WARNING : bad atom id numbers!!!" << endl;
if (id2 >= ar_vector.size()) cout << "WARNING : bad atom id numbers!!!" << endl;
#endif	// VERBOSE

				bondtype tmp_bondtype(bt);
				mm1_bond newbond(ar_vector[id1], ar_vector[id2], tmp_bondtype);
				AddBond(newbond);
				
				p1.getline(buffer, sizeof(buffer));
			}
		}
		else if (!strcmp(buffer, "!Coord"))		// handle "!Coord" section.
		{
			p1.getline(buffer, sizeof(buffer));
			
#ifdef VERBOSE
cout << "found \"!Coord\" section..." << endl;
if (crd_set_counter >= cs_vector.size()) cout << "ERROR : too many crd-sets!!!" << endl;
#endif	// VERBOSE

			while (p1.peek() != '!')
			{
				i32u n1; p1 >> n1;	// id number
				
				fGL xcrd; p1 >> xcrd;	// x-coordinate
				fGL ycrd; p1 >> ycrd;	// y-coordinate
				fGL zcrd; p1 >> zcrd;	// z-coordinate
				
#ifdef VERBOSE
if (n1 >= ar_vector.size()) cout << "ERROR : bad atom id numbers!!!" << endl;
#endif	// VERBOSE

				mm1_atom * ar = ar_vector[n1];
				ar->crd_vector[crd_set_counter][0] = xcrd;
				ar->crd_vector[crd_set_counter][1] = ycrd;
				ar->crd_vector[crd_set_counter][2] = zcrd;
				
				p1.getline(buffer, sizeof(buffer));
			}
			
			crd_set_counter++;
		}
		else if (!strcmp(buffer, "!Charges"))		// handle "!Charges" section.
		{
			p1.getline(buffer, sizeof(buffer));
			
#ifdef VERBOSE
cout << "found \"!Charges\" section..." << endl;
#endif	// VERBOSE

			while (p1.peek() != '!')
			{
				i32u n1; p1 >> n1;	// id number
				f64 n2; p1 >> n2;	// charge
				
#ifdef VERBOSE
if (n1 >= ar_vector.size()) cout << "WARNING : bad atom id numbers!!!" << endl;
#endif	// VERBOSE

				mm1_atom * ar = ar_vector[n1];
				ar->charge = n2;
				
				p1.getline(buffer, sizeof(buffer));
			}
		}
		else						// handle an unknown section...
		{
			p1.getline(buffer, sizeof(buffer));
			
			while (p1.peek() != '!')
			{
				p1.getline(buffer, sizeof(buffer));
			}
		}
	}
	
	return true;
}

// obsolete file output functions are also here, but are commented out. just for reference and for emergency cases...
// obsolete file output functions are also here, but are commented out. just for reference and for emergency cases...
// obsolete file output functions are also here, but are commented out. just for reference and for emergency cases...

/*	// obsolete!!!

// this fileformat is not documented yet... there is not even any identifier
// or any record about the version -> it might be difficult to read these files later.

void mm1_mdl::WriteStream_OLD(ostream & p1)
{
	UpdateIndex();
	
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		p1 << (* it1).index << " " << (* it1).el.GetAtomicNumber() << " ";
		p1 << (* it1).crd_vector[0][0] << " " << (* it1).crd_vector[0][1] << " " << (* it1).crd_vector[0][2] << " ";
		
		vector<i32s> tmp1;	// this is for atomic numbers...
		vector<char> tmp2;	// this is for bond type symbols...
		
		iter_mm1cl it2;
		for (it2 = (* it1).cr_list.begin();it2 != (* it1).cr_list.end();it2++)
		{
			tmp1.push_back((* it2).atmr->index);
			tmp2.push_back((* it2).bndr->bt.GetSymbol1());
		}
		
		p1 << tmp1.size() << ": "; it1++;
		for (i32u n1 = 0;n1 < tmp1.size();n1++)
		{
			p1 << tmp1[n1] << " ";
			p1 << tmp2[n1] << " ";
		}
		
		p1 << endl;
	}
	
	p1 << "END" << endl;
}

*/	// obsolete!!!

// WHAT WE NEED TO ADD WHEN WE CHANGE THE FORMAT NEXT TIME:
// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
// [just add your ideas here]

void mm1_mdl::WriteStream(ostream & p1)
{
	const int current_version_number = 100;		// version number multiplied by 100 -> int.
	
	p1 << "!Header ";
	p1 << GetProjectFileNameExtension() << " ";
	p1 << current_version_number << endl;
	
	// the "!Header" section is now processed, and the actual file output begins!!!
	// the "!Header" section is now processed, and the actual file output begins!!!
	// the "!Header" section is now processed, and the actual file output begins!!!
	
	// write "!Info" section. write the number of coordinate sets,
	// which should match to the number of "!Coord"-sections later in the file.
	
	p1 << "!Info ";
	p1 << cs_vector.size() << endl;
	
// TODO : write visible-flag and set descriptions (if there are some), for example:
// 0 1 "1st coordinate set is this one, and it's visible"
// 1 1 "2nd set looks like this!, and it's also visible"
// 2 0 "this 3rd set is not visible"

	// write "!Atoms" section. write the number of atoms,
	// which should match to the number of records in this section.
	
	p1 << "!Atoms ";
	p1 << atom_list.size() << endl;
	
	UpdateIndex();		// this will update the "index"-records...
	
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		p1 << (* it1).index << " ";
		p1 << (* it1).el.GetAtomicNumber() << " ";
		
		// what else should we print here ?!?!?!?
		// selections?
		
		p1 << endl;
		it1++;
	}
	
	// write "!Bonds" section. write the number of bonds,
	// which should match to the number of records in this section.
	
	p1 << "!Bonds ";
	p1 << bond_list.size() << endl;
	
	iter_mm1bl it2 = bond_list.begin();
	while (it2 != bond_list.end())
	{
		p1 << (* it2).atmr[0]->index << " ";
		p1 << (* it2).atmr[1]->index << " ";
		
		p1 << (* it2).bt.GetSymbol1() << " ";
		
		// what else should we print here ?!?!?!?
		// nothing???
		
		p1 << endl;
		it2++;
	}
	
	// write the "!Coord" sections...
	
	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
	{
		p1 << "!Coord" << endl;
		
		it1 = atom_list.begin();
		while (it1 != atom_list.end())
		{
			p1 << (* it1).index << " ";
			
			p1 << (* it1).crd_vector[n1][0] << " ";
			p1 << (* it1).crd_vector[n1][1] << " ";
			p1 << (* it1).crd_vector[n1][2] << " ";
			
			// what else should we print here ?!?!?!?
			// nothing???
			
			p1 << endl;
			it1++;
		}
	}
	
	// write the !Charges section.
	
	p1 << "!Charges" << endl;
	
	it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		p1 << (* it1).index << " ";
		p1 << (* it1).charge;
		
		p1 << endl;
		it1++;
	}
	
	// write "!End" section.
	
	p1 << "!End" << endl;
}

void mm1_mdl::AddHydrogens(void)
{
	// this will use OELib to add hydrogens!!!
	// this will use OELib to add hydrogens!!!
	// this will use OELib to add hydrogens!!!
	
//	file_trans ft;
//	OEMol * oem = ft.CopyAll(this);
//	oem->AddHydrogens(false, false);
//	ft.Synchronize(); return;
	
////////////////////
////////////////////

	// this is the old implementation...
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		float n1; float n2 = 0;
		switch ((* it1).el.GetAtomicNumber())
		{
			case 6:		n1 = 4.0; break;
			case 7:		n1 = 3.0; break;
			case 8:		n1 = 2.0; break;
			default:	n1 = -1.0;
		}
		
		if (n1 < 0.0) continue;
		
		vector<mm1_atom *> ref;
		for (iter_mm1bl it2 = bond_list.begin();it2 != bond_list.end();it2++)
		{
			if ((* it2).atmr[0] == & (* it1)) ref.push_back((* it2).atmr[1]);
			else if ((* it2).atmr[1] == & (* it1)) ref.push_back((* it2).atmr[0]);
			else continue;
			
			switch ((* it2).bt.GetValue())
			{
				case BONDTYPE_SINGLE:		n2 += 1.0; break;
				case BONDTYPE_CNJGTD:		n2 += 1.5; break;
				case BONDTYPE_DOUBLE:		n2 += 2.0; break;
				case BONDTYPE_TRIPLE:		n2 += 3.0; break;
			        case BONDTYPE_QUADRP:         n2 += 4.0; break;
			}
		}
		
		if ((n1 - 0.1) < n2) continue;
		
		v3d<fGL> v1, v2, v3;
		const fGL ideal = acos(-1.0 / 3.0);
		fGL rnd[3] =
		{
			(fGL) rand() / (fGL) RAND_MAX - 0.5,
			(fGL) rand() / (fGL) RAND_MAX - 0.5,
			(fGL) rand() / (fGL) RAND_MAX - 0.5
		};
		
		switch (ref.size())
		{
			case 0:
			v1 = v3d<fGL>(rnd);
			v1 = v1 * (0.11 / v1.len());
			break;
			
			case 1:
			v1 = v3d<fGL>(rnd);
			v2 = v3d<fGL>((* it1).crd_vector[0].data, ref[0]->crd_vector[0].data);
			v3 = v1.vpr(v2);
			v2 = v2 * (0.11 * cos(ideal) / v2.len());
			v3 = v3 * (0.11 * sin(ideal) / v3.len());
			v1 = v2 + v3;
			break;
			
			case 2:
			v1 = v3d<fGL>((* it1).crd_vector[0].data, ref[0]->crd_vector[0].data);
			v2 = v3d<fGL>((* it1).crd_vector[0].data, ref[1]->crd_vector[0].data);
			v1 = v1 * (1.0 / v1.len());
			v2 = v2 * (1.0 / v2.len());
			v2 = v1 + v2; v3 = v1.vpr(v2);
			v2 = v2 * (0.11 * cos(ideal) / v2.len());
			v3 = v3 * (0.11 * sin(ideal) / v3.len());
			v1 = v2 + v3;
			break;
			
			case 3:
			v1 = v3d<fGL>((* it1).crd_vector[0].data, ref[0]->crd_vector[0].data);
			v2 = v3d<fGL>((* it1).crd_vector[0].data, ref[1]->crd_vector[0].data);
			v3 = v3d<fGL>((* it1).crd_vector[0].data, ref[2]->crd_vector[0].data);
			v1 = v1 * (1.0 / v1.len());
			v2 = v2 * (1.0 / v2.len());
			v3 = v3 * (1.0 / v3.len());
			v1 = v1 + v2; v1 = v1 + v3;
			v1 = v1 * (-0.11 / v1.len());
			break;
		}
		
		fGL tmp[3]; element el = element(1);
		for (i32s n3 = 0;n3 < 3;n3++) tmp[n3] = (* it1).crd_vector[0][n3] + v1.data[n3];
		mm1_atom newatom(el, tmp, cs_vector.size());
		AddAtom(newatom);
		
		bondtype single = bondtype('S');
		mm1_bond newbond(& (* it1), & atom_list.back(), single);
		AddBond(newbond);
		
		// we have to cycle this several times, since we might want
		// to add several hydrogens. change that for-loop -> while-loop ???
		
		it1--;
	}
}

void mm1_mdl::RemoveHydrogens(void)
{
	iter_mm1bl it1 = bond_list.begin();
	while (it1 != bond_list.end())
	{
		bool flag = false;
		if ((* it1).atmr[0]->el.GetAtomicNumber() == 1) flag = true;
		if ((* it1).atmr[1]->el.GetAtomicNumber() == 1) flag = true;
		iter_mm1bl it2 = it1++; if (flag) RemoveBond(it2);
	}
	
	iter_mm1al it2 = atom_list.begin();
	while (it2 != atom_list.end())
	{
		iter_mm1al it3 = it2++;
		if ((* it3).el.GetAtomicNumber() == 1) RemoveAtom(it3);
	}
}

void mm1_mdl::GatherGroups(void)
{
	InvalidateGroups(); iter_mm1al it1;
	nmol = 0; ref_civ = new vector<mm1_chn_info>;
	
	for (it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		(* it1).id[0] = (* it1).id[1] = (* it1).id[2] = (* it1).id[3] = NOT_DEFINED;
	}
	
	while (true)
	{
		for (it1 = atom_list.begin();it1 != atom_list.end() && (* it1).id[0] != NOT_DEFINED;it1++);
		if (it1 != atom_list.end()) GatherAtoms(& (* it1), nmol++); else break;
	}
	
	atom_list.sort();
}

void mm1_mdl::GetRange(i32s ind, i32s value, iter_mm1al * result)
{
	iter_mm1al range[2] = { atom_list.begin(), atom_list.end() };
	GetRange(ind, range, value, result);
}

void mm1_mdl::GetRange(i32s ind, iter_mm1al * range, i32s value, iter_mm1al * result)
{
	result[0] = range[0]; while (result[0] != range[1] && (* result[0]).id[ind] != value) result[0]++;
	result[1] = result[0]; while (result[1] != range[1] && (* result[1]).id[ind] == value) result[1]++;
}

void mm1_mdl::InvalidateGroups(void)
{
	nmol = NOT_DEFINED;
	if (ref_civ != NULL)
	{
		delete ref_civ;
		ref_civ = NULL;
	}
}

i32s mm1_mdl::FindPath(mm1_atom * ref1, mm1_atom * ref2, i32s max, i32s flag, i32s dist)
{
	if (ref1 == ref2) return dist;
	if (dist == max) return NOT_FOUND;
	
	i32s tmp1 = NOT_FOUND; iter_mm1cl it1;
	for (it1 = ref1->cr_list.begin();it1 != ref1->cr_list.end();it1++)
	{
		if ((* it1).bndr->flags[flag]) continue;
		
		(* it1).bndr->flags[flag] = true;
		i32s tmp2 = FindPath((* it1).atmr, ref2, max, flag, dist + 1);
		(* it1).bndr->flags[flag] = false; if (tmp2 < tmp1) tmp1 = tmp2;
	}
	
	return tmp1;
}

bool mm1_mdl::FindRing(mm1_atom * ref1, mm1_atom * ref2, signed char * str, i32s size, i32s flag, i32s dist)
{
	static vector<signed char> ring_vector;
		
	if (!dist && str != NULL) ring_vector.resize(0);
	else if (dist && ref1 == ref2)
	{
		if (dist != size) return false;
		
		if (str != NULL)
		{
			for (i32u n1 = 0;n1 < strlen((const char *) str);n1++)
			{
				if (!(n1 % 2) && str[n1] == '?') continue;
				if ((n1 % 2) && str[n1] == NOT_DEFINED) continue;
				if (str[n1] != ring_vector[n1]) return false;
			}
		}
		
		return true;
	}
	
	if (dist == size) return false; iter_mm1cl it1;
	for (it1 = ref1->cr_list.begin();it1 != ref1->cr_list.end();it1++)
	{
		if ((* it1).bndr->flags[flag]) continue;
		
		if (str != NULL)
		{
			ring_vector.push_back((* it1).bndr->bt.GetSymbol2());
			ring_vector.push_back((signed char) (* it1).atmr->el.GetAtomicNumber());
		}
		
		(* it1).bndr->flags[flag] = true;
		bool result = FindRing((* it1).atmr, ref2, str, size, flag, dist + 1);
		(* it1).bndr->flags[flag] = false;
		
		if (str != NULL)
		{
			ring_vector.pop_back();
			ring_vector.pop_back();
		}
		
		if (result) return true;
	}
	
	return false;
}

void mm1_mdl::UpdateIndex(void)
{
	i32s counter = 0;
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		(* it1++).index = counter++;
	}
}


void mm1_mdl::GatherAtoms(mm1_atom * ref, i32s id)
{
	if (ref->id[0] != NOT_DEFINED) return;
	ref->id[0] = id; iter_mm1cl it1 = ref->cr_list.begin();
	while (it1 != ref->cr_list.end()) GatherAtoms((* it1++).atmr, id);
}

void mm1_mdl::DoEnergy(void)
{
	mm1_eng * eng = CreateDefaultEngine();
	
	CopyCRD(this, eng, 0);
	eng->Compute(0);
	
	char buffer[128];
	ostrstream str(buffer, sizeof(buffer));
	str << "Energy = " << eng->energy << " kJ/mol" << ends;
	err->Message(buffer);
	
	delete eng;
}

void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
{
	GeomOptGetParam(param);
	if (!param.confirm) return;
	
	mm1_eng * eng = CreateDefaultEngine();

	CopyCRD(this, eng, 0);
	
	mm1_geomopt * opt = new mm1_geomopt(eng, 100, 0.025);		// optimal settings?!?!?
	
	for (i32s n1 = 0;n1 < param.nsteps;n1++)
	{
		opt->TakeCGStep(conjugate_gradient::Newton2An);
		cout << n1 << " " << opt->optval << " " << opt->optstp << " " << endl;
		
		if (!(n1 % 10))
		{
			CopyCRD(eng, this, 0); CenterCRDSet(0);
			UpdateAllGraphicsViews(true);
		}
	}
	
	delete opt;
	delete eng;
}

void mm1_mdl::GeomOptGetParam(mm1_geomopt_param & param)
{
	param.confirm = true;
}

#define CFRQ 1
#define CSIZE 100

#define START 10000

void mm1_mdl::DoMolDyn(mm1_moldyn_param & param)
{
	MolDynGetParam(param);
	if (!param.confirm) return;
	
	mm1_eng * eng = CreateDefaultEngine();
	
	mm1_eng_exp1b3 * yyy = NULL;		// temporary...
	mm1_eng_pbc * zzz = NULL;		// temporary...
	
	mm1_eng_exp1_mim * xxx = dynamic_cast<mm1_eng_exp1_mim *>(eng);
	if (xxx != NULL) zzz = yyy = xxx;
	
	CopyCRD(this, eng, 0);
	
	mm1_moldyn * dyn = new mm1_moldyn(eng, param.temperature, param.timestep);
	
	ofstream ofile;
	ofile.open(param.filename, ios::out | ios::binary);
	
	const char file_id[10] = "traj_v100";
	const int number_of_atoms = atom_list.size();
	const int total_frames = param.nsteps / 100;
	
	ofile.write((char *) file_id, 8);					// file id, 8 chars.
	ofile.write((char *) & number_of_atoms, sizeof(number_of_atoms));	// number of atoms, int.
	ofile.write((char *) & total_frames, sizeof(total_frames));		// total number of frames, int.
	
	i32s ccnt = 0; f64 control[CSIZE];
	
	for (i32s n1 = 0;n1 < START + param.nsteps;n1++)
	{
		if (!(n1 % 1000) && zzz != NULL) zzz->CheckLocations();
		if (!(n1 % 100) && yyy != NULL) yyy->update = true;
		
		dyn->TakeMDStep();
		if (!(n1 % 50))
		{
			cout << n1 << " T = " << dyn->ConvEKinTemp(dyn->kin);
			cout << " pot = " << dyn->pot << " tot = " << (dyn->kin + dyn->pot) << endl;
		}
		
		if (n1 < START)
		{
			if (!(n1 % CFRQ))
			{
				control[ccnt++] = dyn->kin;
			}
			
			if (ccnt == CSIZE)
			{
				f64 sum = 0.0;
				for (i32s n9 = 0;n9 < ccnt;n9++)
				{
					sum += control[n9];
				}
				
				sum /= (f64) ccnt; ccnt = 0; f64 rtemp = (f64) (n1 + CFRQ) / (f64) START;
				cout << "average = " << sum << " kJ/mol = " << dyn->ConvEKinTemp(sum) << " K." << endl;
				dyn->SetEKin(dyn->kin + (dyn->ConvTempEKin(dyn->temp * rtemp) - sum));
			}
		}
		else if (!(n1 % 100))
		{
			CopyCRD(eng, this, 0);
			
			const float ekin = dyn->kin;
			const float epot = dyn->pot;
			
			ofile.write((char *) & ekin, sizeof(ekin));	// kinetic energy, float.
			ofile.write((char *) & epot, sizeof(epot));	// potential energy, float.
			
			for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				for (i32s t4 = 0;t4 < 3;t4++)		// all coordinates, float.
				{
					float t1a = (* it1).crd_vector[0][t4];
					ofile.write((char *) & t1a, sizeof(t1a));
				}
			}
		}
		
		if (!(n1 % 100))
		{
			CopyCRD(eng, this, 0);
			UpdateAllGraphicsViews(true);
		}
	}
	
	delete dyn;
	delete eng;
	
	UpdateAccumValues();	// is this needed !??!?!!?
}

void mm1_mdl::MolDynGetParam(mm1_moldyn_param & param)
{
	param.confirm = true;
}

void mm1_mdl::OpenTrajectory(const char * fn)
{
	if (!trajfile)
	{
		trajfile = new ifstream(fn, ios::in | ios::binary);
		
		trajfile->seekg(8, ios::beg);	// skip the file id...
		
		int natoms;
		trajfile->read((char *) & natoms, sizeof(natoms));
		
		if (natoms != (int) atom_list.size())
		{
			cout << "incompatible file : different number of atoms!" << endl;
			CloseTrajectory(); return;
		}
		
		trajfile->read((char *) & total_frames, sizeof(total_frames));
		cout << "the trajectory file contains " << total_frames << " frames." << endl;
		
		current_frame = 0;
	}
	else cout << "trajectory already exists!" << endl;
}

void mm1_mdl::CloseTrajectory(void)
{
	if (trajfile != NULL)
	{
		trajfile->close();
		delete trajfile;
		trajfile = NULL;
	}
}

void mm1_mdl::ReadFrame(void)
{
	i32s atoms = atom_list.size();
	
	i32s place = 8 + 2 * sizeof(int);				// skip the header...
	place += (2 + 3 * atoms) * sizeof(float) * current_frame;	// get the correct frame...
	place += 2 * sizeof(float);					// skip epot and ekin...
	
	trajfile->seekg(place, ios::beg);
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s t4 = 0;t4 < 3;t4++)
		{
			float t1a; trajfile->read((char *) & t1a, sizeof(t1a));
			(* it1).crd_vector[0][t4] = t1a;
		}
	}
}

/*################################################################################################*/

// eof
