// MM2MDL.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "mm2mdl.h"
#include "mm1mdl.h"

#include <algorithm>

/*################################################################################################*/

mm2_mdl::mm2_mdl(ostream * p1, err_util_factory & p2, void *) :
	model_extended(p1, p2), model_simple(p1, p2)
{
}

mm2_mdl::~mm2_mdl(void)
{
}

const char * mm2_mdl::GetProjectFileNameExtension(void)
{
	static const char ext[] = "mm2gp";
	return ext;
}

bool mm2_mdl::ReadStream(istream & istr)
{
	i32s ncs; istr >> buffer >> ncs;
	PushCRDSets(ncs - cs_vector.size());
	
	i32s n1; i32s n2; i32s n3; i32s n4;
	char tmp1; char tmp2; fGL tmp3;
	
	while (istr.peek() != '#')
	{
		if (istr.peek() == 'c')		// chn
		{
			mm2_chn newchn;
			chn_vector.push_back(newchn);
		}
		
		if (istr.peek() == 'r')		// res
		{
			istr >> buffer >> n1 >> tmp1 >> tmp2 >> tmp3;
			istr.getline(buffer, sizeof(buffer));
			
			mm2_res newres = mm2_res(tmp1, cs_vector.size(), NULL);
			
			switch (tmp2)
			{
				case 'H':	newres.state = STATE_HELIX; break;
				case 'S':	newres.state = STATE_STRAND; break;
				default:	newres.state = STATE_LOOP;
			}
			
			for (n1 = 0;n1 < ncs;n1++) newres.peptide_vector[n1] = tmp3;
			
			for (n1 = 0;n1 < (i32s) newres.natm;n1++)
			{
				for (n2 = 0;n2 < ncs;n2++)
				{
					istr >> tmp3; newres.crd_vector[n1][n2][0] = tmp3;
					istr >> tmp3; newres.crd_vector[n1][n2][1] = tmp3;
					istr >> tmp3; newres.crd_vector[n1][n2][2] = tmp3;
				}
				
				if (n1 + 1 == (i32s) newres.natm) continue;
				else istr.getline(buffer, sizeof(buffer));
			}
			
			chn_vector.back().res_vector.push_back(newres);
		}
		
		if (istr.peek() == 'B')		// bridge
		{
			mm2_dsb newdsb; istr.get();
			istr >> n1 >> n2 >> n3 >> n4;
			
			newdsb.chn[0] = n1; newdsb.res[0] = n2;
			newdsb.chn[1] = n3; newdsb.res[1] = n4;
			
			dsb_vector.push_back(newdsb);
		}
		
		istr.getline(buffer, sizeof(buffer));
	}
	
//	UpdateConstraints();	// PREVIEW...

	return true;
}

void mm2_mdl::WriteStream(ostream & ostr)
{
	ostr << "ncs " << cs_vector.size() << endl;
	
	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
	{
		ostr << "chn " << n1 << endl;
		
		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
		{
			ostr << "res " << n2 << " ";
			ostr << chn_vector[n1].res_vector[n2].symbol << " ";
			
			switch (chn_vector[n1].res_vector[n2].state)
			{
				case STATE_HELIX:	ostr << "H"; break;
				case STATE_STRAND:	ostr << "S"; break;
				default:		ostr << "L";
			} ostr << " ";
			
			for (i32u n3 = 0;n3 < cs_vector.size();n3++)
			{
				ostr << chn_vector[n1].res_vector[n2].peptide_vector[n3] << endl;
			}
			
			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
			{
				for (i32u n4 = 0;n4 < cs_vector.size();n4++)
				{
					ostr << chn_vector[n1].res_vector[n2].crd_vector[n3][n4][0] << " ";
					ostr << chn_vector[n1].res_vector[n2].crd_vector[n3][n4][1] << " ";
					ostr << chn_vector[n1].res_vector[n2].crd_vector[n3][n4][2] << " ";
				} ostr << endl;
			}
		}
		
		ostr << "endchn" << endl;
	}
	
	for (i32u n1 = 0;n1 < dsb_vector.size();n1++)
	{
		ostr << "B ";
		ostr << dsb_vector[n1].chn[0] << " ";
		ostr << dsb_vector[n1].res[0] << " ";
		ostr << dsb_vector[n1].chn[1] << " ";
		ostr << dsb_vector[n1].res[1] << endl;
	}
	
	ostr << "#end" << endl;
}

void mm2_mdl::PushCRDSets(i32u p1)
{
	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());
	
	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
	{
		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
		{
			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
			{
				for (i32u n4 = 0;n4 < p1;n4++)
				{
					chn_vector[n1].res_vector[n2].crd_vector[n3].push_back(newcrd);
				}
			}
			
			for (i32u n3 = 0;n3 < p1;n3++)
			{
				chn_vector[n1].res_vector[n2].peptide_vector.push_back(0.0);
			}
		}
	}
}

void mm2_mdl::PopCRDSets(i32u)
{
	// not yet implemented...
	// not yet implemented...
	// not yet implemented...
}

void mm2_mdl::CopyCRDSet(i32u p1, i32u p2)
{
	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
	{
		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
		{
			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
			{
				chn_vector[n1].res_vector[n2].crd_vector[n3][p2][0] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][0];
				chn_vector[n1].res_vector[n2].crd_vector[n3][p2][1] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][1];
				chn_vector[n1].res_vector[n2].crd_vector[n3][p2][2] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][2];
			}
			
			chn_vector[n1].res_vector[n2].peptide_vector[p2] = chn_vector[n1].res_vector[n2].peptide_vector[p1];
		}
	}
}

void mm2_mdl::SwapCRDSets(i32u, i32u)
{
	// not yet implemented...
	// not yet implemented...
	// not yet implemented...
}

void mm2_mdl::CenterCRDSet(i32u)
{
	// not yet implemented...
	// not yet implemented...
	// not yet implemented...
}

void mm2_mdl::ReserveCRDSets(i32u)
{
}

/*################################################################################################*/

// eof
