// QM1ENG1.H : the MPQC front-end is implemented here.

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "config.h"	// this is target-dependent...

#ifndef QM1ENG1_H
#define QM1ENG1_H

#ifdef ENABLE_MPQC

class qm1_eng_mpqc;

/*################################################################################################*/

#include "qm1eng.h"
#include "qm1mdl.h"

#include <fstream>

#include "util/keyval/keyval.h"			// KeyVal

#include "util/group/message.h"			// MessageGrp
#include "util/group/thread.h"			// ThreadGrp

#include "chemistry/molecule/energy.h"		// MolecularEnergy
#include "chemistry/qc/wfn/obwfn.h"		// OneBodyWavefunction
#include "chemistry/qc/wfn/wfn.h"		// Wavefunction

#include "math/scmat/abstract.h"		// SCMatrix

/*################################################################################################*/

/**	The MPQC "wrapper" engine class...

	Strategy: 
	Initialize using a standard MPQC input file -> we will have also an input file 
	generator. For each different situation (closed shell, open shell, whatever) we 
	will have a different class. This is only for closed shell cases -> split this 
	to a base class/derived classes in future!!!!! For example, open shell cases 
	might have separate matrices for alpha and beta orbitals etc... 
*/

class qm1_eng_mpqc : public qm1_eng
{
	protected:
	
	static i32u ifile_name_counter;
	
	RefMessageGrp grp;
	RefThreadGrp thread;
	
	RefParsedKeyVal parsedkv;
	RefKeyVal keyval;
	
	RefMolecularEnergy mole;
	
	RefOneBodyWavefunction obwfn;		// for Hartree-Fock orbitals...
	
	RefWavefunction wfn;			// for natural orbitals...
	RefSCMatrix natorb;			// for natural orbitals...
	
	public:
	
	qm1_eng_mpqc(qm1_mdl &);
	~qm1_eng_mpqc(void);
	
	void GetInputFileName(char *, i32u);
	void WriteInputFile(ofstream &);
	
	const f64 * GetCRD(i32u);		// virtual
	void PutCRD(i32u, const f64 *);		// virtual
	
	/**	It seems that we get the energy in atomic units... A set of unit conversion 
		functions should be made that makes conversions si-units <-> atomic units <-> 
		whatever units. These engine classes should have some "states" which determine 
		units that are printed out ?!?!?!?!?!?
	*/
	
	void Compute(i32s);		// virtual
	void Check(i32s);		// virtual
	
	fGL GetESP(fGL *, fGL *);		// virtual
	fGL GetElDens(fGL *, fGL *);		// virtual
	
	fGL GetOrbital(fGL *, fGL *);		// virtual
	fGL GetOrbDens(fGL *, fGL *);		// virtual
};

/*################################################################################################*/

/**	This is just a draft; And it seems this is a bad draft...

	Strategy: 
	Create the MPQC-engine inside this class and append all optimizer-stuff to 
	the input file. So it seems we can't simply follow the MM idea of engine -> 
	algorithms here !??!?! On the other hand, we could make exactly similar 
	algorithms here like in MM section, but those will less efficient if 
	there is a "native" counterpart.
*/

class qm1_mpqc_optimizer	// base class?!?!?!?
{
	protected:
	
	qm1_eng_mpqc * eng;
	
	public:
	
	qm1_mpqc_optimizer(qm1_mdl &);
	~qm1_mpqc_optimizer(void);
};

/*################################################################################################*/

#endif	// ENABLE_MPQC
#endif	// QM1ENG1_H

// eof
