// QM1MDL.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "qm1mdl.h"	// config.h is here -> we get ENABLE-macros here...

#include "mm1mdl.h"	// for import...

#include "qm1eng1.h"
#include "qm1eng9.h"

#include <strstream>
#include <fstream>

/*################################################################################################*/

qm1_atom::qm1_atom(void)
{
}

qm1_atom::qm1_atom(element p1, fGL * p2, i32u p3)
{
	el = p1;
	
	fGL_a3 data = { p2[0], p2[1], p2[2] };
	for (i32u n1 = 0;n1 < p3;n1++) crd_vector.push_back(data);
}

qm1_atom::qm1_atom(const qm1_atom & p1)
{
	el = p1.el;
	crd_vector = p1.crd_vector;
}

qm1_atom::~qm1_atom(void)
{
}

// for qm1_atoms, equality is tested using pointers -> you can find the
// iterator (using STL's find-function) if the pointer is known.

bool qm1_atom::operator==(const qm1_atom & p1) const
{
	return (this == (& p1));
}

/*################################################################################################*/

qm1_mdl::qm1_mdl(ostream * p1, err_util_factory & p2) : model_extended(p1, p2), model_simple(p1, p2)
{
	current_eng = NULL;
	current_orbital = 0;
	
	default_eng = 0;
}

qm1_mdl::~qm1_mdl(void)
{
	if (current_eng != NULL) delete current_eng;
}

const char * qm1_mdl::GetProjectFileNameExtension(void)
{
	static const char ext[] = "qm1gp";
	return ext;
}

/*##############################################*/
/*##############################################*/

const char * qm1_mdl::engtab1[] =
{

#ifdef ENABLE_MPQC

	"qm1_eng_mpqc : the MPQC engine",
	
#endif	// ENABLE_MPQC

	"qm1_eng_hf1 : a non-functional dummy engine",
	NULL
};

const i32s qm1_mdl::engtab2[] =
{

#ifdef ENABLE_MPQC

	ENG_QM1_MPQC,
	
#endif	// ENABLE_MPQC

	ENG_QM1_DUMMY,
	NOT_DEFINED
};

qm1_eng * qm1_mdl::CreateDefaultEngine(void)
{
	switch (engtab2[default_eng])
	{

#ifdef ENABLE_MPQC

		case ENG_QM1_MPQC:
		return new qm1_eng_mpqc(* this);
		
#endif	// ENABLE_MPQC

		default:	// ENG_QM1_DUMMY
		return new qm1_eng_hf1(* this);
	}
}

void qm1_mdl::DiscardCurrentEng(void)
{
	if (current_eng != NULL)
	{
		delete current_eng;
		current_eng = NULL;
	}
}

/*##############################################*/
/*##############################################*/

// this stuff is pretty much stolen from the mm1_mdl...
// this stuff is pretty much stolen from the mm1_mdl...
// this stuff is pretty much stolen from the mm1_mdl...

void qm1_mdl::PushCRDSets(i32u p1)
{
//	cs_vector.resize(cs_vector.size() + p1);			// more convenient way?!?!?!
//	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back();		// more crash-proof way?!?!?!?
	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());	// or even this way?!?!?!?
	
	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
	for (iter_qm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32u n1 = 0;n1 < p1;n1++)
		{
			(* it1).crd_vector.push_back(newcrd);
		}
	}
}

void qm1_mdl::PopCRDSets(i32u)
{
}

void qm1_mdl::CopyCRDSet(i32u, i32u)
{
}

void qm1_mdl::SwapCRDSets(i32u, i32u)
{
}

void qm1_mdl::CenterCRDSet(i32u p1)
{
	fGL sum[3] = { 0.0, 0.0, 0.0 };
	for (iter_qm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++) sum[n1] += (* it1).crd_vector[p1][n1];
	}
	
	for (iter_qm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			fGL tmp1 = sum[n1] / (f64) atom_list.size();
			(* it1).crd_vector[p1][n1] -= tmp1;
		}
	}
}

void qm1_mdl::ReserveCRDSets(i32u)
{
}

void qm1_mdl::AddAtom(qm1_atom & p1)
{
	DiscardCurrentEng();
	atom_list.push_back(p1);
}

void qm1_mdl::RemoveAtom(iter_qm1al it1)
{
	DiscardCurrentEng();
	atom_list.erase(it1);
}

void qm1_mdl::ImportMM1(const char * fn)
{
	mm1_mdl * mdl = new mm1_mdl(NULL, * err_factory);
	
	ifstream ifile(fn, ios::in);
	mdl->ReadStream(ifile);
	ifile.close();
	
	iter_mm1al it1 = mdl->atom_list.begin();
	while (it1 != mdl->atom_list.end())
	{
		element tmp1 = (* it1).el;
		fGL * tmp2 = (* it1).crd_vector[0].data;
		
		qm1_atom newatom(tmp1, tmp2, cs_vector.size());
		AddAtom(newatom);
		
		it1++;
	}
	
	delete mdl;
}

void qm1_mdl::DoEnergy(void)
{
	if (!current_eng) current_eng = CreateDefaultEngine();
	
	CopyCRD(this, current_eng, 0); current_eng->Compute(0);
	
	char buffer[128];
	ostrstream str(buffer, sizeof(buffer));
	str << "Energy = " << current_eng->energy << " Hartrees" << ends;	// the energy unit is for MPQC...
	err->Message(buffer);
	
	// we will not delete current_eng here, so that we can draw plots using it...
	// we will not delete current_eng here, so that we can draw plots using it...
	// we will not delete current_eng here, so that we can draw plots using it...
}

/*################################################################################################*/

// eof
