// QM1MDL.H : model class for molecular QM models...

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "config.h"	// this is target-dependent...

#ifndef QM1MDL_H
#define QM1MDL_H

class qm1_mdl;

/*################################################################################################*/

#include "model.h"

#include "qm1eng.h"
#include "utility.h"

#include <list>
#include <vector>

/*################################################################################################*/

class qm1_atom
{
//	protected:
	public:		// heavily used...
	
	element el;
	vector<a3_fGL> crd_vector;
	
	public:
	
	qm1_atom(void);
	qm1_atom(element, fGL *, i32u);
	qm1_atom(const qm1_atom &);
	~qm1_atom(void);
	
	bool operator==(const qm1_atom &) const;	// using pointers...
};

typedef list<qm1_atom>::iterator iter_qm1al;		// al = atom list

/*################################################################################################*/

#define ENG_QM1_MPQC		0x10		// qm1_eng_mpqc
#define ENG_QM1_DUMMY		0x11		// qm1_eng_hf1

/**	A "##model" class for molecular quantum mechanics models.
	
	My hope is that a simple set of QM methods would be implemented in this package, 
	for educational purposes if nothing else... But it's a very VERY demanding job, 
	which will take at least a decade, or most likely forever, to complete.
	
	So, now we have to borrow that code outside. this design is mostly an adaption 
	to the MPQC program, where we currently borrow our QM energy/gradient code...
*/

class qm1_mdl : public model_extended
{
	protected:
	
	list<qm1_atom> atom_list;
	
	qm1_eng * current_eng;
	
	i32s current_orbital;
	
	// default engine settings begin!!!
	// default engine settings begin!!!
	// default engine settings begin!!!
	
	public:
	
	static const char * engtab1[];
	static const i32s engtab2[];
	
	i32s default_eng;
	
	// default engine settings end!!!
	// default engine settings end!!!
	// default engine settings end!!!
	
	friend class qm1_eng;
	friend class qm1_eng_mpqc;
	
	friend void CopyCRD(qm1_mdl *, qm1_eng *, i32u);
	friend void CopyCRD(qm1_eng *, qm1_mdl *, i32u);
	
	public:
	
	qm1_mdl(ostream *, err_util_factory &);
	~qm1_mdl(void);
	
	const char * GetProjectFileNameExtension(void);		// virtual
	
	qm1_eng * CreateDefaultEngine(void);
	virtual void DiscardCurrentEng(void);
	
	bool ReadStream(istream &) { return false; }	// virtual
	void WriteStream(ostream &) { }			// virtual
	
	void PushCRDSets(i32u);			// virtual
	void PopCRDSets(i32u);			// virtual
	
	void CopyCRDSet(i32u, i32u);		// virtual
	void SwapCRDSets(i32u, i32u);		// virtual
	
	void CenterCRDSet(i32u);		// virtual
	void ReserveCRDSets(i32u);		// virtual
	
	void AddAtom(qm1_atom &);
	void RemoveAtom(iter_qm1al);
	
	void ImportMM1(const char *);
	
	public:
	
	// here we have a set of Do???()-functions. the idea is that there is a set
	// of features that we wish to behave EXACTLY same way in each target/platform.
	// we then create a Do???()-function for the feature, and hide the details of
	// the user interface in a set of virtual functions.

	/// This will perform an energy calculation.
	
	void DoEnergy(void);
};

/*################################################################################################*/

#endif	// QM1MDL_H

// eof
