// CAMERA.CPP

// Copyright (C) 1998 Tommi Hassinen, Jarno Huuskonen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "camera.h"	// config.h is here -> we get ENABLE-macros here...

#ifdef ENABLE_GRAPHICS
#include "docview.h"
#include "views.h"

/*################################################################################################*/

camera::camera(const object_location & p1, fGL p2, docview * p3) : dummy_object(p1)
{
	focus = p2; docv = p3;
	use_local_lights = true;
	use_global_lights = true;
	ortho = false;
	
	GetLocDataRW()->crd[2] = -focus;
}

// this is only partial because only camera::OrbitObject() needs this!!!!!!!
// this is only partial because only camera::OrbitObject() needs this!!!!!!!
// this is only partial because only camera::OrbitObject() needs this!!!!!!!

camera::camera(const camera & p1) : dummy_object(* p1.ol)
{
}

camera::~camera(void)
{
}

// when rotating or translating a camera we must make all operations in reversed directions
// and also drag all those local lights along...

void camera::OrbitObject(const fGL * ang, const camera & cam)
{
	fGL tmp_ang[3];
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		tmp_ang[n1] = -ang[n1];
	}
	
	for (i32u n1 = 0;n1 < docv->light_vector.size();n1++)
	{
		if (docv->light_vector[n1]->owner != this) continue;
		docv->light_vector[n1]->OrbitObject(tmp_ang, cam);
	}
	
	dummy_object::OrbitObject(tmp_ang, cam);
	
	DoCameraEvents();
}

// when rotating a camera, we must also make the local lights to orbit around the camera.
// dummy_object::OrbitObject() will orbit the lights around the camera's focus point, so here
// we must set the camera focus to zero before making the transformation...

void camera::RotateObject(const fGL * ang, const camera & cam)
{
	fGL tmp_ang[3];
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		tmp_ang[n1] = -ang[n1];
	}
	
	camera tmp_cam = cam; tmp_cam.focus = 0.0;
	for (i32u n1 = 0;n1 < docv->light_vector.size();n1++)
	{
		if (docv->light_vector[n1]->owner != this) continue;
		docv->light_vector[n1]->OrbitObject(tmp_ang, tmp_cam);
	}
	
	dummy_object::RotateObject(tmp_ang, cam);
	
	DoCameraEvents();
}

void camera::TranslateObject(const fGL * dst, const obj_loc_data * data)
{
	fGL tmp_dst[3];
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		tmp_dst[n1] = -dst[n1];
	}
	
	for (i32u n1 = 0;n1 < docv->light_vector.size();n1++)
	{
		if (docv->light_vector[n1]->owner != this) continue;
		docv->light_vector[n1]->TranslateObject(tmp_dst, data);
	}
	
	dummy_object::TranslateObject(tmp_dst, data);
	
	DoCameraEvents();
}

void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
{
	if (gv->size[0] < 0) return;
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	if (rm == Draw || rm == Erase || rm == Select)
	{
		glRenderMode(GL_SELECT);
		iGLs vp[4]; glGetIntegerv(GL_VIEWPORT, vp);
		
		// is this dependent on screen resolution or what???
		// there are some problems with selection in wireframe models...
		
		gluPickMatrix(x, vp[3] - y, 2.0, 2.0, vp);	// is 2.0 ok???
	}
	
	fGL aspect = (fGL) gv->size[0] / (fGL) gv->size[1];
	fGL fovy = (aspect > 1.0 ? 45.0 / aspect : 45.0);
	
	gv->range[1] = focus * tan(M_PI * fovy / 360.0);
	gv->range[0] = aspect * gv->range[1];
	
	if (!ortho) gluPerspective(fovy, aspect, 0.1 * focus, 2.0 * focus);
	else glOrtho(-gv->range[0], +gv->range[0], -gv->range[1], +gv->range[1], 0.0, 2.0 * focus);
	
	const obj_loc_data * loc1 = GetLocData();
	
	glMatrixMode(GL_MODELVIEW); glLoadIdentity(); v3d<fGL> target = v3d<fGL>(loc1->crd) + (loc1->zdir * focus);
	gluLookAt(loc1->crd[0], loc1->crd[1], loc1->crd[2], target[0], target[1], target[2], loc1->ydir[0], loc1->ydir[1], loc1->ydir[2]);
	
	for (i32u n1 = 0;n1 < gv->GetDV()->light_vector.size();n1++)
	{
		camera * owner = gv->GetDV()->light_vector[n1]->owner;
		if (owner != NULL && owner != this) continue;
		gv->GetDV()->light_vector[n1]->SetupLocation();
	}
	
	glInitNames();
	if (transformer::transform_in_progress)
	{
		gv->GetDV()->Render(gv, docview::Transform1);
		glPushMatrix(); gv->current_tool->ref_to_object->SetModelView();
		gv->GetDV()->Render(gv, docview::Transform2); glPopMatrix();
	}
	else gv->GetDV()->Render(gv, docview::Normal);	// do the normal rendering!!!
	
	// in TARGET2, print out the tool name and the selected object (if any)...
	// in TARGET2, print out the tool name and the selected object (if any)...
	// in TARGET2, print out the tool name and the selected object (if any)...
	
#ifdef TARGET2

	if (gv->draw_info && rm == Normal)
	{
		glColor3f(1.0, 1.0, 1.0);
		glDisable(GL_DEPTH_TEST);
		
		glPushMatrix();
		glLoadIdentity();
		
		glMatrixMode(GL_PROJECTION);
		glPushMatrix(); glLoadIdentity();
		gluOrtho2D(0, gv->size[0], 0, gv->size[1]);
		
		// print out the selected object, if there is any...
		// print out the selected object, if there is any...
		// print out the selected object, if there is any...
		
		void * font = GLUT_BITMAP_TIMES_ROMAN_10;
		const char * txt = "[no selected object]"; fGL xpos; fGL ypos;
		if (gv->GetDV()->selected_object) txt = gv->GetDV()->selected_object->GetObjectName();
		
		xpos = (gv->size[0] - gv->GetBitmapStringWidth(font, txt)) / 2.0;
		ypos = 5.0;
		
		glRasterPos2f(xpos, ypos); gv->WriteBitmapString(font, txt);
		
		// print out the current tool...
		// print out the current tool...
		// print out the current tool...
		
		txt = gv->current_tool->GetToolName();
		
		xpos = (gv->size[0] - gv->GetBitmapStringWidth(font, txt)) / 2.0;
		ypos = gv->size[1] - 10.0;
		
		glRasterPos2f(xpos, ypos); gv->WriteBitmapString(font, txt);
		
		// ok...
		// ok...
		// ok...
		
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		
		glPopMatrix();
		glEnable(GL_DEPTH_TEST);
	}
	
#endif	// TARGET2

	if (rm == Draw || rm == Erase || rm == Select)
	{
		i32s tmp1 = glRenderMode(GL_RENDER);
		i32s tmp2 = NOT_DEFINED;
		
	//	iGLu tmp3 = 0xffffffff;		// old, lowest z-value...
		iGLu tmp3 = 0x00000000;
		
		// here we examine the selection buffer and select the hit with lowest
		// z-value (since we assume that this object was visible to the user).
		
	// HAS THIS SELECTION-LOGIC CHANGED LATELY?!?!?! THIS WAS DONE AT 1998 "BY THE BOOK"
	// AND I THINK IT WORKED CORRECTLY. BUT NOW THE OLD CODE SEEMS TO PICK ALWAYS FROM
	// BACK, NOT FROM FRONT?!?!?! NOW IT'S FIXED AND USES THE HIGHEST Z-VALUE...
	
		i32s tmp4[2] = { 0, 0 };
		while (tmp4[0] < tmp1)
		{
	//		if (gv->GetDV()->select_buffer[tmp4[1] + 1] < tmp3)	// old, lowest z-value...
			if (gv->GetDV()->select_buffer[tmp4[1] + 2] > tmp3)
			{
				tmp2 = tmp4[1];
	//			tmp3 = gv->GetDV()->select_buffer[tmp4[1] + 1];	// old, lowest z-value...
				tmp3 = gv->GetDV()->select_buffer[tmp4[1] + 2];
			}
			
			tmp4[0]++;
			tmp4[1] += gv->GetDV()->select_buffer[tmp4[1]] + 3;
		}
		
		// now we copy all name records of the selected hit...
		
		vector<iGLu> name_vector;
		if (tmp2 != NOT_DEFINED)
		{
			for (i32u i1 = 0;i1 < gv->GetDV()->select_buffer[tmp2];i1++)
			{
				name_vector.push_back(gv->GetDV()->select_buffer[tmp2 + i1 + 3]);
			}
		}
		
		// "draw"- and "erase"-events are always forwarded to the docv-object:
		// "draw"- and "erase"-events are always forwarded to the docv-object:
		// "draw"- and "erase"-events are always forwarded to the docv-object:
		
		if (rm == Draw) gv->GetDV()->DrawEvent(gv, name_vector);
		if (rm == Erase) gv->GetDV()->EraseEvent(gv, name_vector);
		
		// "select"-events are forwarded only if they are MODEL_DEPENDENT:
		// "select"-events are forwarded only if they are MODEL_DEPENDENT:
		// "select"-events are forwarded only if they are MODEL_DEPENDENT:
		
		if (rm == Select && name_vector.size() > 1)
		{
			bool test = (name_vector[0] & GLNAME_MODEL_DEPENDENT);
			if (test)
			{
				if (gv->GetDV()->selected_object != NULL)
				{
					gv->GetDV()->selected_object = NULL;
					gv->GetDV()->event_SelectedObjectChanged();
				}
				
				gv->GetDV()->SelectEvent(gv, name_vector);
			}
			else
			{
				switch (name_vector[0])
				{
					case GLNAME_LIGHT:
					gv->GetDV()->SelectLight((const dummy_object *) name_vector[1]);
					break;
					
					case GLNAME_OBJECT:
					gv->GetDV()->SelectObject((const dummy_object *) name_vector[1]);
					break;
				}
			}
		}
	}
}

// * how to use clipping planes (to make comparison easier) ???
// * how to use clipping planes (to make comparison easier) ???
// * how to use clipping planes (to make comparison easier) ???

void camera::RenderObjects(graphics_view * gv)
{
	for (i32u n1 = 0;n1 < gv->GetDV()->light_vector.size();n1++)
	{
		camera * owner = gv->GetDV()->light_vector[n1]->owner;
		if (owner != NULL && owner != this) continue;
		gv->GetDV()->light_vector[n1]->Render();
	}
	
	for (i32u n1 = 0;n1 < gv->GetDV()->object_vector.size();n1++)
	{
		if (gv->GetDV()->object_vector[n1]->transparent) continue;
		gv->GetDV()->object_vector[n1]->Render();
	}
}

void camera::DoCameraEvents(void)
{
	list<smart_object *>::iterator so_it;
	for (so_it = obj_list.begin();so_it != obj_list.end();so_it++)
	{
		(* so_it)->CameraEvent(* this);
	}
}

/*################################################################################################*/

const fGL light::def_amb_comp[4] = { 0.1, 0.1, 0.1 };
const fGL light::def_diff_comp[4] = { 0.8, 0.8, 0.8 };
const fGL light::def_spec_comp[4] = { 0.8, 0.8, 0.8 };

const light_components light::def_components =
{
	(fGL *) light::def_amb_comp,
	(fGL *) light::def_diff_comp,
	(fGL *) light::def_spec_comp
};

light::light(const object_location & p1, const light_components & p2) : dummy_object(p1)
{
	owner = NULL;
	number = NOT_DEFINED;
	InitComponents(& p2);
}

light::~light(void)
{
	delete[] amb_comp;
	delete[] diff_comp;
	delete[] spec_comp;
}

void light::InitComponents(const light_components * p1)
{
	amb_comp = new fGL[4];
	diff_comp = new fGL[4];
	spec_comp = new fGL[4];
	
	for (i32s n1 = 0;n1 < 4;n1++)
	{
		amb_comp[n1] = p1->amb_comp[n1];
		diff_comp[n1] = p1->diff_comp[n1];
		spec_comp[n1] = p1->spec_comp[n1];
	}
}

/*################################################################################################*/

fGL spot_light::size = 0.05;

fGL spot_light::shade1[3] = { 0.0, 0.0, 0.7 };
fGL spot_light::shade2[3] = { 0.0, 0.0, 0.9 };

fGL spot_light::bulb_on[3] = { 1.0, 1.0, 0.5 };
fGL spot_light::bulb_off[3] = { 0.5, 0.5, 0.5 };

const fGL spot_light::def_cutoff = 45.0;
const fGL spot_light::def_exponent = 32.0;

spot_light::spot_light(const object_location & p1, const light_components & p2, fGL p3, fGL p4) : light(p1, p2)
{
	GetLocDataRW()->crd[3] = 1.0;		// make this a spotlight!!!
	cutoff = p3; exponent = p4;
}

spot_light::~spot_light(void)
{
}

void spot_light::SetupProperties(void)
{
	glLightf((GLenum) number, GL_SPOT_CUTOFF, cutoff);
	glLightf((GLenum) number, GL_SPOT_EXPONENT, exponent);
	
	glLightfv((GLenum) number, GL_AMBIENT, amb_comp);
	glLightfv((GLenum) number, GL_DIFFUSE, diff_comp);
	glLightfv((GLenum) number, GL_SPECULAR, spec_comp);
}

void spot_light::SetupLocation(void)
{
	glLightfv((GLenum) number, GL_POSITION, GetLocData()->crd);
	glLightfv((GLenum) number, GL_SPOT_DIRECTION, GetLocData()->zdir.data);
}

#define SHADE_ANGLE (22.5 / 180.0 * M_PI)

void spot_light::Render(void)
{
	fGL r1 = tan(SHADE_ANGLE) * size;
	fGL tmp1 = 1.0 / (sin(SHADE_ANGLE) + 1.0);
	fGL tmp2 = 1.2 * size;
	
	fGL r2 = (1.0 - tmp1) * tmp2;
	fGL r3 = tmp1 * tmp2;
	
	const dummy_object * name = this;
	glPushName(GLNAME_LIGHT); glPushName((iGLu) name);
	
	glPushMatrix(); SetModelView();
	
	if (cutoff < 180.0)
	{
		glPushMatrix();
		glTranslatef(0.0, 0.0, -r3);
		glBegin(GL_TRIANGLES);
		
		for (i32s n1 = 0;n1 < 8;n1++)
		{
			if (n1 & 1) glColor3fv(shade1);
			else glColor3fv(shade2);
			
			fGL ang1 = M_PI * (fGL) n1 / 4.0;
			fGL ang2 = M_PI * (fGL) (n1 + 1) / 4.0;
			
			glVertex3f(r1 * cos(ang1), r1 * sin(ang1), size);
			glVertex3f(r1 * cos(ang2), r1 * sin(ang2), size);
			glVertex3f(0.0, 0.0, 0.0);
		}
		
		glEnd();	// GL_TRIANGLES
		glPopMatrix();
	}
	
	if (glIsEnabled((GLenum) number)) glColor3fv(bulb_on);
	else glColor3fv(bulb_off);
	
	GLUquadricObj * qo = gluNewQuadric();
	gluQuadricDrawStyle(qo, (GLenum) GLU_FILL);
	gluSphere(qo, 0.95 * r2, 8, 4);
	gluDeleteQuadric(qo);
	
	glPopMatrix();
	
	glPopName();
	glPopName();
}

/*################################################################################################*/

directional_light::directional_light(const object_location & p1, const light_components & p2) : light(p1, p2)
{
}

directional_light::~directional_light(void)
{
}

void directional_light::SetupProperties(void)
{
	glLightf((GLenum) number, GL_SPOT_CUTOFF, 180.0);
	glLightf((GLenum) number, GL_SPOT_EXPONENT, 0.0);
	
	glLightfv((GLenum) number, GL_AMBIENT, amb_comp);
	glLightfv((GLenum) number, GL_DIFFUSE, diff_comp);
	glLightfv((GLenum) number, GL_SPECULAR, spec_comp);
}

// the direction of a directional light is the reversed z-axis direction!!!
// the direction of a directional light is the reversed z-axis direction!!!
// the direction of a directional light is the reversed z-axis direction!!!

void directional_light::SetupLocation(void)
{
	fGL tmp_crd[4]; tmp_crd[3] = 0.0;	// make this a directional light!!!
	
	for (i32s n1 = 0;n1 < 3;n1++) tmp_crd[n1] = -GetLocData()->zdir[n1];
	glLightfv((GLenum) number, GL_POSITION, tmp_crd);
}

void directional_light::Render(void)
{
}

/*################################################################################################*/

bool transformer::transform_in_progress = false;

transformer::transformer() : dummy_object(true)
{
	target_docv = NULL;
}

transformer::~transformer(void)
{
}

void transformer::Init(docview * p1)
{
	target_docv = p1;
	
	obj_loc_data * data = GetLocDataRW();
	for (i32s n1 = 0;n1 < 3;n1++) data->crd[n1] = 0.0;
	data->zdir = v3d<fGL>(0.0, 0.0, 1.0);
	data->ydir = v3d<fGL>(0.0, 1.0, 0.0);
}

void transformer::GetMatrix(fGL * p1) const
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix(); glLoadIdentity();
	SetModelView(); glGetFloatv(GL_MODELVIEW_MATRIX, p1);
	glPopMatrix();
}

bool transformer::BeginTransformation(void)
{
	transform_in_progress = true;
	target_docv->Center(this);
	return true;
}

bool transformer::EndTransformation(void)
{
	transform_in_progress = false;
	target_docv->Transform(this);
	return true;
}

void transformer::OrbitObject(const fGL * p1, const camera & p2)
{
	dummy_object::RotateObject(p1, p2);
}

void transformer::RotateObject(const fGL * p1, const camera & p2)
{
	dummy_object::OrbitObject(p1, p2);
}

void transformer::Render(void)
{
}

/*################################################################################################*/

#endif	// ENABLE_GRAPHICS

// eof
