// QM1GUI.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "qm1docv.h"	// config.h is here -> we get ENABLE-macros here...

#ifdef ENABLE_GRAPHICS
#include "qm1eng1.h"
#include "qm1eng9.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "views.h"

/*################################################################################################*/

qm1_cm_element qm1_docv::cm_element = qm1_cm_element();

qm1_docv::qm1_docv(view_factory & p1, ostream * p2, err_util_factory & p3) :
	docview(p1, p2, p3), qm1_mdl(p2, p3), model_simple(p2, p3)
{
}

qm1_docv::~qm1_docv(void)
{
}

void qm1_docv::DiscardCurrentEng(void)
{
	i32u n1 = 0;
	while (n1 < object_vector.size())
	{
		bool flag = false;
		
		color_plane_object * cpo_ref = dynamic_cast<color_plane_object *>(object_vector[n1]);
		if (cpo_ref != NULL && cpo_ref->GetRef() == current_eng) flag = true;
		
		volume_rendering_object * vro_ref = dynamic_cast<volume_rendering_object *>(object_vector[n1]);
		if (vro_ref != NULL && vro_ref->GetRef() == current_eng) flag = true;
		
		color_surface_object * cso_ref = dynamic_cast<color_surface_object *>(object_vector[n1]);
		if (cso_ref != NULL && cso_ref->GetRef() == current_eng) flag = true;
		
		if (flag)
		{
			if (selected_object == object_vector[n1]) selected_object = NULL;
			RemoveObject(object_vector[n1]);
		}
		else n1++;
	}
	
	qm1_mdl::DiscardCurrentEng();
}

fGL qm1_docv::GetDefaultFocus(void)
{
	return 2.0;
}

const char * qm1_docv::GetType(void)
{
	return "QM, molecular calculations";
}

color_mode * qm1_docv::GetDefaultColorMode(void)
{
	return & qm1_docv::cm_element;
}

void qm1_docv::SelectAll(void)
{
}

void qm1_docv::InvertSelection(void)
{
}

// it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!

void qm1_docv::InitGL(void)
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
	
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	const fGL specular_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	const fGL ambient_intensity[4] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
	
	glFogi(GL_FOG_MODE, GL_EXP); glFogf(GL_FOG_DENSITY, 0.15);
	const fGL fog_color[4] = { 0.0, 0.0, 0.0, 0.0 };
	glFogfv(GL_FOG_COLOR, fog_color);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glSelectBuffer(SB_SIZE, select_buffer);
}

void qm1_docv::Render(graphics_view * gv, rmode rm)
{
	bool accum = gv->accumulate; if (rm != Normal) accum = false;
	if (accum) { glClear(GL_ACCUM_BUFFER_BIT); UpdateAccumValues(); }
	else if (rm != Transform2) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	if (gv->enable_fog) glEnable(GL_FOG);
	
	/*################*/
	/*################*/
	
	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
	{
		if (!cs_vector[n1].visible) continue;
		if (accum) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		if (gv->render == RENDER_WIREFRAME)
		{
			glPointSize(3.0); glLineWidth(1.0);
			for (iter_qm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				glPushName(GLNAME_MD_TYPE1);
				glPushName((i32u) & (* it1));
				
				glBegin(GL_POINTS);
				
				fGL_a4 color;
				gv->colormode->GetColor(& (* it1), color);
				glColor3fv(color);

				glVertex3fv((* it1).crd_vector[n1].data);
				glEnd();
				
				glPopName();
				glPopName();
			}
		}
		
		if (gv->render != RENDER_WIREFRAME && gv->render != RENDER_NOTHING)
		{
			glEnable(GL_LIGHTING);
			
			for (iter_qm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				fGL_a4 color;
				gv->colormode->GetColor(& (* it1), color);
				glColor3fv(color);
				
				float rad = (* it1).el.GetVDWRadius() * 0.125;
				int res = 12;
				
				glPushName(GLNAME_MD_TYPE1); glPushName((i32u) & (* it1));
				
				GLUquadricObj * qo = gluNewQuadric();
				gluQuadricDrawStyle(qo, (GLenum) GLU_FILL);
				glPushMatrix();
				glTranslated((* it1).crd_vector[n1][0], (* it1).crd_vector[n1][1], (* it1).crd_vector[n1][2]);
				gluSphere(qo, rad, res, res / 2);
				glPopMatrix();
				gluDeleteQuadric(qo);
				
				glPopName(); glPopName();
			}
			
			glDisable(GL_LIGHTING);
		}
		
		if (accum)
		{
			gv->cam->RenderObjects(gv);
			glAccum(GL_ACCUM, cs_vector[n1].accum_value);
		}
	}
	
	/*################*/
	/*################*/
	
	if (accum) glAccum(GL_RETURN, 1.0);
	else if (rm != Transform2) gv->cam->RenderObjects(gv);
	
//	if (gv->label == LABEL_ELEMENT)
//	{
//		glDisable(GL_DEPTH_TEST); glColor3f(1.0, 1.0, 1.0); char string[32];
//		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
//		{
//			glRasterPos3fv((* it1).crd_vector[0].data);
//			
//			ostrstream str(string, sizeof(string));
//			str << (* it1).el.GetSymbol() << ends;
//			gv->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
//		}
//		glEnable(GL_DEPTH_TEST);  
//	}
	
	if (gv->enable_fog) glDisable(GL_FOG);
	
	// finally call this to handle transparency...
	// finally call this to handle transparency...
	// finally call this to handle transparency...
	
	RenderAllTPs(gv, rm);
}

void qm1_docv::Center(transformer *)
{
}

void qm1_docv::Transform(transformer *)
{
}

// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...

void qm1_docv::DrawEvent(graphics_view * gv, vector<iGLu> & names)
{
	if (ogl_view::button == mouse_tool::Right) return;	// the right button is for popup menus...
	
	i32s mouse[2] =
	{
		gv->current_tool->latest_x,
		gv->current_tool->latest_y
	};
	
	if (ogl_view::state == mouse_tool::Down)
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[0] = (qm1_atom *) names[1];
		}
		else
		{
			fGL tmp1[3]; gv->GetCRD(mouse, tmp1);
			qm1_atom newatom(element::current_element, tmp1, cs_vector.size());
			AddAtom(newatom); draw_data[0] = & atom_list.back();
		}
	}
	else
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[1] = (qm1_atom *) names[1];
		}
		else
		{
			fGL tmp1[3]; gv->GetCRD(mouse, tmp1);
			qm1_atom newatom(element::current_element, tmp1, cs_vector.size());
			AddAtom(newatom); draw_data[1] = & atom_list.back();
		}
		
		// if different: update bondtype or add a new bond.
		// if not different: change atom to different element.
		
		if (draw_data[0] != draw_data[1])
		{
//			mm1_bond newbond(draw_data[0], draw_data[1], bondtype::current_bondtype);
//			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), newbond);
//			if (it1 != bond_list.end()) (* it1).bt = bondtype::current_bondtype;
//			else AddBond(newbond);
		}
		else
		{
			draw_data[0]->el = element::current_element;
		}
		
		UpdateAllGraphicsViews();
	}
}

// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...

void qm1_docv::EraseEvent(graphics_view * gv, vector<iGLu> & names)
{
	if (ogl_view::button == mouse_tool::Right) return;	// the right button is for popup menus...
	
	if (ogl_view::state == mouse_tool::Down)
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[0] = (qm1_atom *) names[1];
		}
		else
		{
			draw_data[0] = NULL;
		}
	}
	else
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[1] = (qm1_atom *) names[1];
		}
		else
		{
			draw_data[1] = NULL;
		}
		
		if (!draw_data[0] || !draw_data[1]) return;
		
		// if different: try to find and remove a bond.
		// if not different: remove atom.
		
		if (draw_data[0] != draw_data[1])
		{
//			mm1_bond tmpbond(draw_data[0], draw_data[1], bondtype::current_bondtype);
//			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), tmpbond);
//			if (it1 != bond_list.end()) RemoveBond(it1); else return;
		}
		else
		{
			iter_qm1al it1 = find(atom_list.begin(), atom_list.end(), (* draw_data[0]));
			if (it1 != atom_list.end()) RemoveAtom(it1); else exit(EXIT_FAILURE);
		}
		
		UpdateAllGraphicsViews();
	}
}

void qm1_docv::SelectEvent(graphics_view *, vector<iGLu> &)
{
}

/*################################################################################################*/

void qm1_cm_element::GetColor(const void * p1, fGL_a4 & p2)
{
	qm1_atom * ref = (qm1_atom *) p1;
	const fGL * color = ref->el.GetColor();
	for (i32s n1 = 0;n1 < 3;n1++) p2[n1] = color[n1];
}

/*################################################################################################*/

#endif	// ENABLE_GRAPHICS

// eof
