// QM2DOCV.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "qm2docv.h"	// config.h is here -> we get ENABLE-macros here...

#ifdef ENABLE_GRAPHICS
#include "plane.h"
#include "surface.h"

#include "color.h"
#include "views.h"

/*################################################################################################*/

qm2_docv::qm2_docv(view_factory & p1, ostream * p2, err_util_factory & p3) :
	docview(p1, p2, p3), qm2_mdl(p2, p3), model_simple(p2, p3)
{
}

qm2_docv::~qm2_docv(void)
{
}

fGL qm2_docv::GetDefaultFocus(void)
{
	return 1.0;
}

const char * qm2_docv::GetType(void)
{
	return "QM, atomic calculations";
}

color_mode * qm2_docv::GetDefaultColorMode(void)
{
	return NULL;
}

void qm2_docv::SelectAll(void)
{
}

void qm2_docv::InvertSelection(void)
{
}

// it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!

void qm2_docv::InitGL(void)
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
	
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	const fGL specular_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	const fGL ambient_intensity[4] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
	
	glFogi(GL_FOG_MODE, GL_EXP); glFogf(GL_FOG_DENSITY, 0.15);
	const fGL fog_color[4] = { 0.0, 0.0, 0.0, 0.0 };
	glFogfv(GL_FOG_COLOR, fog_color);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glSelectBuffer(SB_SIZE, select_buffer);
}

void qm2_docv::Render(graphics_view * gv, rmode rm)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	if (gv->enable_fog) glEnable(GL_FOG);
	
	glPointSize(3.0); glLineWidth(1.0);
	
	glBegin(GL_POINTS);
	glColor3fv(el.GetColor());
	glVertex3f(0.0, 0.0, 0.0);
	glEnd();
	
	if (rm != Transform2) gv->cam->RenderObjects(gv);
	if (gv->enable_fog) glDisable(GL_FOG);
	
	// finally call this to handle transparency...
	// finally call this to handle transparency...
	// finally call this to handle transparency...
	
	RenderAllTPs(gv, rm);
}

void qm2_docv::Center(transformer *)
{
}

void qm2_docv::Transform(transformer *)
{
}

void qm2_docv::DrawEvent(graphics_view *, vector<iGLu> &)
{
}

void qm2_docv::EraseEvent(graphics_view *, vector<iGLu> &)
{
}

void qm2_docv::SelectEvent(graphics_view *, vector<iGLu> &)
{
}

/*################################################################################################*/

#endif	// ENABLE_GRAPHICS

// eof
