// T2VIEWS.H : a view window class for GLUT.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "config.h"	// this is for TARGET2...

#ifndef T2VIEWS_H
#define T2VIEWS_H

class glut_ogl_view;
class glut_graphics_view;

class glut_view_factory;

/*################################################################################################*/

#include "views.h"
#include "t2docv.h"

#include <vector>

/*################################################################################################*/

class glut_view : virtual public view
{
	protected:
	
	glut_docv * docv;
	
	i32s glut_id;
	
	public:
	
	glut_view(glut_docv *);
	virtual ~glut_view(void);
	
	docview * GetDV(void);		// virtual
};

/*################################################################################################*/

class glut_ogl_view : public glut_view, virtual public ogl_view
{
	private:
	
	static vector<glut_ogl_view *> ogl_vector;
	
	public:
	
	glut_ogl_view(glut_docv *);
	virtual ~glut_ogl_view(void);
	
	void Update(bool);			// virtual
	void SetTitle(const char *);		// virtual
	
	void SetCurrent(void);			// virtual
	
	private:
	
	virtual void DisplayEvent(void) = 0;
	virtual void KeyboardEvent(unsigned char, int, int) = 0;
	virtual void SpecialEvent(int, int, int) = 0;
	
	private:
	
	static glut_ogl_view * GetWindow(int);
	
	static void DisplayHandler(void);
	static void KeyboardHandler(unsigned char, int, int);
	static void SpecialHandler(int, int, int);
	
	static void ReshapeHandler(int, int);
	
	static void MouseHandler(int, int, int, int);
	static void MotionHandler(int, int);
};

/*################################################################################################*/

class glut_graphics_view : public glut_ogl_view, public graphics_view
{
	private:
	
	glut_graphics_view(glut_docv *, camera *);
	
	friend class glut_view_factory;
	
	public:
	
	~glut_graphics_view(void);
	
	private:
	
	void DisplayEvent(void);				// virtual
	void KeyboardEvent(unsigned char, int, int);		// virtual
	void SpecialEvent(int, int, int);			// virtual
};

/*################################################################################################*/

class glut_view_factory : public view_factory
{
	private:
	
	static glut_view_factory * instance;
	glut_view_factory(void);
	
	public:
	
	~glut_view_factory(void);
	static glut_view_factory * GetInstance(void);
	
	graphics_view * ProduceGraphicsView(docview *, camera *);	// virtual
};

/*################################################################################################*/

#endif	// T2VIEWS_H

// eof
