// T3MM2DOCV.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "t3mm2docv.h"

#include "t3main.h"
#include "t3dialog.h"
#include "t3errutil.h"
#include "t3views.h"

#include "mm2rbn.h"

#include "color.h"
#include "views.h"

#include "mm1mdl.h"

#include <fstream>
#include <strstream>

/*################################################################################################*/

const char mm2_file_open_dialog::title[] = "Open Project File - mm2gp";
mm2_docv * mm2_file_open_dialog::docv = NULL;

mm2_file_open_dialog::mm2_file_open_dialog(mm2_docv * p1) : file_dialog(title, NULL, NULL)
{
	docv = p1;
}

mm2_file_open_dialog::~mm2_file_open_dialog(void)
{
}

bool mm2_file_open_dialog::OkEvent(const char * filename)
{
	if (docv == NULL)
	{
		docv = new gnome_mm2_docv(* gnome_view_factory::GetInstance(), & cout, * gnome_err_util_factory::GetInstance(), NULL);
		docv->AddGraphicsView();
	}
	
	ifstream ifile;
	ifile.open(filename, ios::in);
	docv->ReadStream(ifile);
	ifile.close();
	
	docv->ParseProjectFileNameAndPath(filename);
	docv->UpdateAllWindowTitles();
	
	docv->UpdateAllGraphicsViews();
	
	return true;
}

void mm2_file_open_dialog::CancelEvent(void)
{
}

/*################################################################################################*/

GnomeUIInfo gnome_mm2_docv::file_submenu[] =
{
	GNOMEUIINFO_MENU_CLOSE_ITEM(popup_FileClose, NULL),
	
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::select_submenu[] =
{
	GNOMEUIINFO_ITEM("Select All", "select all atoms...",
		gnome_graphics_view::popup_SelectAll, NULL),
		
	GNOMEUIINFO_ITEM("Invert Selection", "invert the selection...",
		gnome_graphics_view::popup_InvertSelection, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::viewscreate_submenu[] =
{
	GNOMEUIINFO_ITEM("New Camera", "create a new view with a new camera...",
		gnome_graphics_view::popup_ViewsNewCam, NULL),
		
	GNOMEUIINFO_ITEM("New View", "create a new view using the current camera...",
		gnome_graphics_view::popup_ViewsNewView, NULL),
		
//	GNOMEUIINFO_SEPARATOR,
//	
//	GNOMEUIINFO_ITEM("TreeList", "create a tree/list view...",
//		popup_TreeList, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::views_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Create", viewscreate_submenu),
	
	GNOMEUIINFO_ITEM("Delete This View", "remove this view...",
		gnome_graphics_view::popup_ViewsDeleteView, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::lights_submenu[] =
{
	GNOMEUIINFO_ITEM("Create New Light", "create a new light object...",
		gnome_graphics_view::popup_LightsNewLight, NULL),
		
	GNOMEUIINFO_SEPARATOR,
	
	GNOMEUIINFO_TOGGLEITEM("Display Local Lights", "loc",
		gnome_graphics_view::popup_LightsSwitchLoc, NULL),
		
	GNOMEUIINFO_TOGGLEITEM("Display Global Lights", "glob",
		gnome_graphics_view::popup_LightsSwitchGlob, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::rendermode_style_radio[] =
{
	GNOMEUIINFO_ITEM("Ball & Stick", "bs",
		popup_RModeBallAndStick, NULL),
		
	GNOMEUIINFO_ITEM("Wireframe", "wf",
		popup_RModeWireFrame, NULL),
		
	GNOMEUIINFO_ITEM("van der Waals", "vdw",
		popup_RModeVanDerWaals, NULL),
		
	GNOMEUIINFO_ITEM("Solvent-Accessible", "sas",
		popup_RModeAccessible, NULL),
		
	GNOMEUIINFO_ITEM("Cylinders", "cyl",
		popup_RModeCylinders, NULL),
		
	GNOMEUIINFO_ITEM("Nothing", "null",
		popup_RModeNothing, NULL),

	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::rendermode_submenu[] =
{
        GNOMEUIINFO_RADIOLIST(rendermode_style_radio),
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::projection_radio[] =
{
	GNOMEUIINFO_ITEM("Perspective", "persp",
		gnome_graphics_view::popup_ProjPerspective, NULL),
		
	GNOMEUIINFO_ITEM("Orthographic", "ortho",
		gnome_graphics_view::popup_ProjOrthographic, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::projection_submenu[] =
{
        GNOMEUIINFO_RADIOLIST(projection_radio),
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::render_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Views", views_submenu),
	GNOMEUIINFO_SUBTREE("_Lights", lights_submenu),
	GNOMEUIINFO_SUBTREE("_Rendering Mode", rendermode_submenu),
	GNOMEUIINFO_SUBTREE("_Projection", projection_submenu),
	
	GNOMEUIINFO_TOGGLEITEM("Quick Update Mode", "quick update...",
		gnome_graphics_view::popup_RenderQuickUpdate, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::objects_submenu[] =
{
	GNOMEUIINFO_ITEM("Ribbon", "...",
		popup_ObjRibbon, NULL),
	
	GNOMEUIINFO_SEPARATOR,
	
	GNOMEUIINFO_ITEM("Delete Current Object", "delete",
		gnome_graphics_view::popup_ObjectsDeleteCurrent, NULL),
		
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::popupmenu_info[] =
{
	GNOMEUIINFO_SUBTREE("_File", file_submenu),
	GNOMEUIINFO_SUBTREE("_Select", select_submenu),
	
	GNOMEUIINFO_SEPARATOR,
	
	GNOMEUIINFO_SUBTREE("_Render", render_submenu),
	GNOMEUIINFO_SUBTREE("_Objects", objects_submenu),
	
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_mm2_docv::popupmenu_dummy[] =
{
        GNOMEUIINFO_SUBTREE("Ghemical", popupmenu_info),
        GNOMEUIINFO_END
};

/*################################################################################################*/

gnome_mm2_docv::gnome_mm2_docv(view_factory & p1, ostream * p2, err_util_factory & p3, void * p4) :
	gnome_docv(p1, p2, p3), mm2_docv(p1, p2, p3, p4), docview(p1, p2, p3), model_simple(p2, p3)
{
	popupmenu = gnome_popup_menu_new(popupmenu_info);
	
	// all the rest stuff here is gnome-mdi related...
	// all the rest stuff here is gnome-mdi related...
	// all the rest stuff here is gnome-mdi related...
	
	// if we wish to use gnome_mdi_restore_state or whatever those "state saving" features are,
	// we must pack this stuff into a static (callback) function. but we have to handle different
	// document types (as well as different view types) and gnome 1.X API seems to have poor
	// support for that. so we leave all that for future improvements...
	
	child = gnome_mdi_generic_child_new(GetFullProjectFileName());
	gtk_object_set_user_data(GTK_OBJECT(child), (docview *) this);
	
	gtk_signal_connect(GTK_OBJECT(child), "destroy", GTK_SIGNAL_FUNC(gnome_mdi_app::DestroyChildHandler), NULL);
	
	gnome_mdi_generic_child_set_config_func(child, GetConfigString, NULL);
	gnome_mdi_generic_child_set_view_creator(child, gnome_view_factory::ViewCreator, NULL);
	
//	gnome_mdi_child_set_menu_template(GNOME_MDI_CHILD(child), ???);		// we use the popup-menu...
	
	gnome_mdi_add_child(gnome_mdi_app::GetInstance()->mdi, GNOME_MDI_CHILD(child));
}

gnome_mm2_docv::~gnome_mm2_docv(void)
{
}

// FileClose is always the same -> move to gnome_graphics_view ?!?!?!?
// FileClose is always the same -> move to gnome_graphics_view ?!?!?!?
// FileClose is always the same -> move to gnome_graphics_view ?!?!?!?

void gnome_mm2_docv::popup_FileClose(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gnome_mdi_remove_child(gnome_mdi_app::mdi, GNOME_MDI_CHILD(gv->docv->child), FALSE);
}

void gnome_mm2_docv::popup_RModeWireFrame(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gv->render = RENDER_WIREFRAME; gv->docv->UpdateAllGraphicsViews();
}

void gnome_mm2_docv::popup_RModeBallAndStick(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gv->render = RENDER_BALL_AND_STICK; gv->docv->UpdateAllGraphicsViews();
}

void gnome_mm2_docv::popup_RModeVanDerWaals(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gv->render = RENDER_VAN_DER_WAALS; gv->docv->UpdateAllGraphicsViews();
}

void gnome_mm2_docv::popup_RModeAccessible(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gv->render = RENDER_ACCESSIBLE; gv->docv->UpdateAllGraphicsViews();
}

void gnome_mm2_docv::popup_RModeCylinders(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gv->render = RENDER_CYLINDERS; gv->docv->UpdateAllGraphicsViews();
}

void gnome_mm2_docv::popup_RModeNothing(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gv->render = RENDER_NOTHING; gv->docv->UpdateAllGraphicsViews();
}

void gnome_mm2_docv::popup_ObjRibbon(GtkWidget *, gpointer data)
{
	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
	if (docv)
	{
		for (i32s n1 = 0;n1 < (i32s) docv->chn_vector.size();n1++)
		{
			smart_object * NewObject;
			NewObject = new mm2_ribbon(docv, gv->colormode, n1, 4);		// min. order is 2!!!
			docv->AddObject(NewObject);
		}
		
		docv->UpdateAllGraphicsViews();
	}
}

/*################################################################################################*/

// eof
