// T3VIEWS.H : a view window class for GTK+/GNOME.

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

/*################################################################################################*/

#include "config.h"	// this is for TARGET3...

#ifndef T3VIEWS_H
#define T3VIEWS_H

class gnome_ogl_view;
class gnome_graphics_view;

class gnome_treelist_view;

class gnome_view_factory;

/*################################################################################################*/

#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>
#include <gnome.h>

#include "views.h"
#include "t3docv.h"

#include <vector>

/*################################################################################################*/

/// GNOME-dependent part of "##view" class.

class gnome_view : virtual public view
{
	protected:
	
	gnome_docv * docv;
	
	GtkWidget * view_widget;
	
	public:
	
	gnome_view(gnome_docv *);
	virtual ~gnome_view(void);
	
	docview * GetDV(void);		// virtual
};

/*################################################################################################*/

class gnome_ogl_view : public gnome_view, virtual public ogl_view
{
	private:
	
	static vector<gnome_ogl_view *> ogl_vector;
	
	friend class gnome_graphics_view;
	
	public:
	
	gnome_ogl_view(gnome_docv *);
	virtual ~gnome_ogl_view(void);
	
	void Update(bool);			// virtual
	void SetTitle(const char *);		// virtual
	
	void SetCurrent(void);			// virtual
	
	private:
	
	virtual void ExposeEvent(void) = 0;
	
	private:
	
	static gnome_ogl_view * GetWindow(GtkWidget *);
	
	static void RealizeHandler(GtkWidget *, gpointer);
	static gint ExposeHandler(GtkWidget *, GdkEventExpose *);
	static gint ButtonHandler(GtkWidget *, GdkEventButton *);
	static gint MotionNotifyHandler(GtkWidget *, GdkEventMotion *);
	static gint ConfigureHandler(GtkWidget *, GdkEventConfigure *);
};

/*################################################################################################*/

/// A GNOME-version of "##ogl_view".

class gnome_graphics_view : public gnome_ogl_view, public graphics_view
{
	private:
	
	gnome_graphics_view(gnome_docv *, camera *);
	
	friend class gnome_view_factory;
	
	friend class gnome_qm1_docv;
	friend class gnome_mm1_docv;
	friend class gnome_mm2_docv;
		
	public:
	
	~gnome_graphics_view(void);
	
	// the model-independent popup-menu callbacks are here...
	// the model-independent popup-menu callbacks are here...
	// the model-independent popup-menu callbacks are here...
	
	static void popup_SelectAll(GtkWidget *, gpointer);
	static void popup_InvertSelection(GtkWidget *, gpointer);

	static void popup_ProjOrthographic(GtkWidget *, gpointer);
	static void popup_ProjPerspective(GtkWidget *, gpointer);
	
	static void popup_RenderQuickUpdate(GtkWidget *, gpointer);

	static void popup_ViewsNewCam(GtkWidget *, gpointer);
	static void popup_ViewsNewView(GtkWidget *, gpointer);
	static void popup_ViewsDeleteView(GtkWidget *, gpointer);
	
	static void popup_LightsNewLight(GtkWidget *, gpointer);
	static void popup_LightsSwitchLoc(GtkWidget *, gpointer);
	static void popup_LightsSwitchGlob(GtkWidget *, gpointer);

	static void popup_ObjectsDeleteCurrent(GtkWidget *, gpointer);
	
	private:
	
	static gnome_graphics_view * GetGV(GtkWidget *);
	
	void ExposeEvent(void);		// virtual
};

/*################################################################################################*/

// these records will always share at least the item.
// a "record" base class?!?!?!? how much would it help???

struct gnome_tlv_light_record
{
	GtkWidget * item;
	light * ref;
};

struct gnome_tlv_object_record
{
	GtkWidget * item;
	smart_object * ref;
};

/**	A GNOME treelist view implementation. Under construction, 
	but still offers a way to select some objects which can't be selected otherwise.
	
	The rumors seem to indicate that tree and/or list widgets are still evolving rapidly in GTK. 
	The tree widget used here is an older one, so this implementation is likely to change...
	
	Also this tree/list system is quite complicated, and needs some more careful desing...
*/

class gnome_treelist_view : public gnome_view, public treelist_view
{
	protected:

	static vector<gnome_treelist_view *> tl_vector;
	
	vector<gnome_tlv_light_record> light_vector;
	vector<gnome_tlv_object_record> object_vector;
	
	GtkWidget * item_lights;
	GtkWidget * subt_lights;
	GtkWidget * item_objects;
	GtkWidget * subt_objects;
	GtkWidget * tree;
	
	GtkWidget * list;
	
	private:
	
	gnome_treelist_view(gnome_docv *);
	
	friend class gnome_view_factory;
		
	public:
	
	~gnome_treelist_view(void);
	
	void Update(bool);			// virtual
	void SetTitle(const char *);		// virtual
	
	void LightAdded(light *);	// virtual
	void LightRemoved(light *);	// virtual
	
	void ObjectAdded(smart_object *);	// virtual
	void ObjectRemoved(smart_object *);	// virtual
	
	static void SelectChildHandler(GtkTree *, GtkWidget *, gpointer);
	void SelectChildEvent(GtkTree *, GtkWidget *);
};

/*################################################################################################*/

// gnome 1.x API does not actively support different view types, so we must (try to) implement
// it here ourselves... we use the static "view_type" variable to determine the view type.

#define VIEWTYPE_DEFAULT	0
#define VIEWTYPE_GRAPHICS	101
#define VIEWTYPE_TREELIST	102

class gnome_view_factory : public view_factory
{
	private:
	
	static gnome_view_factory * instance;
	gnome_view_factory(void);
	
	static i32s view_type;
	
	static camera * last_cam;
	static gnome_graphics_view * last_gv;
	static gnome_treelist_view * last_tlv;
	
	public:
	
	~gnome_view_factory(void);
	static gnome_view_factory * GetInstance(void);
	
//	static GnomeMDIChild * ChildCreator(const char *);
	static GtkWidget * ViewCreator(GnomeMDIChild *, gpointer);
	
	graphics_view * ProduceGraphicsView(docview *, camera *);	// virtual
	treelist_view * ProduceTreeListView(docview *);			// virtual
};

/*################################################################################################*/

#endif  // T3VIEWS_H

// eof
